@@ -66,21 +66,6 @@ struct ppm_device {
 	wait_queue_head_t read_queue;
 };
 
-/*
- * The ring descriptor.
- * We have one of these for each CPU.
- */
-struct ppm_ring_buffer_context {
-	bool open;
-	bool capture_enabled;
-	struct ppm_ring_buffer_info *info;
-	char *buffer;
-	struct timespec last_print_time;
-	u32 nevents;
-	atomic_t preempt_count;
-	char *str_storage;	/* String storage. Size is one page. */
-};
-
 /*
  * FORWARD DECLARATIONS
  */
@@ -88,13 +73,22 @@ static int ppm_open(struct inode *inode, struct file *filp);
 static int ppm_release(struct inode *inode, struct file *filp);
 static long ppm_ioctl(struct file *f, unsigned int cmd, unsigned long arg);
 static int ppm_mmap(struct file *filp, struct vm_area_struct *vma);
-static void record_event(enum ppm_event_type event_type,
+static int record_event_consumer(struct ppm_consumer_t *consumer,
+	enum ppm_event_type event_type,
 	struct pt_regs *regs,
 	long id,
-	int never_drop,
+	enum syscall_flags drop_flags,
+	struct task_struct *sched_prev,
+	struct task_struct *sched_next,
+	struct timespec *ts);
+static void record_event_all_consumers(enum ppm_event_type event_type,
+	struct pt_regs *regs,
+	long id,
+	enum syscall_flags drop_flags,
 	struct task_struct *sched_prev,
 	struct task_struct *sched_next);
-
+static int init_ring_buffer(struct ppm_ring_buffer_context *ring);
+static void free_ring_buffer(struct ppm_ring_buffer_context *ring);
 static ssize_t ppe_write(struct file *filp, const char __user *buf, size_t count, loff_t *f_pos);
 
 #ifndef CONFIG_HAVE_SYSCALL_TRACEPOINTS
@@ -134,15 +128,8 @@ static const struct file_operations g_ppe_fops = {
 /*
  * GLOBALS
  */
-
-static DEFINE_PER_CPU(struct ppm_ring_buffer_context*, g_ring_buffers);
-static DEFINE_MUTEX(g_open_mutex);
-static u32 g_open_count;
-u32 g_snaplen = RW_SNAPLEN;
-u32 g_sampling_ratio = 1;
-static u32 g_sampling_interval;
-static int g_is_dropping;
-static int g_dropping_mode;
+LIST_HEAD(g_consumer_list);
+static DEFINE_MUTEX(g_consumer_mutex);
 static bool g_tracepoint_registered;
 
 struct cdev *g_ppe_cdev = NULL;
@@ -161,6 +148,8 @@ static bool verbose = 1;
 static bool verbose = 0;
 #endif
 
+static unsigned int max_consumers = 5;
+
 #define vpr_info(fmt, ...)					\
 do {								\
 	if (verbose)						\
@@ -186,39 +175,165 @@ static void compat_unregister_trace(void *func, const char *probename, struct tr
 #endif
 }
 
+static struct ppm_consumer_t *ppm_find_consumer(struct task_struct *consumer_id)
+{
+	struct ppm_consumer_t *el = NULL;
+
+	rcu_read_lock();
+	list_for_each_entry_rcu(el, &g_consumer_list, node) {
+		if (el->consumer_id == consumer_id) {
+			rcu_read_unlock();
+			return el;
+		}
+	}
+	rcu_read_unlock();
+
+	return NULL;
+}
+
+static void check_remove_consumer(struct ppm_consumer_t *consumer, int remove_from_list)
+{
+	int cpu;
+	int open_rings = 0;
+
+	for_each_online_cpu(cpu) {
+		struct ppm_ring_buffer_context *ring = per_cpu_ptr(consumer->ring_buffers, cpu);
+		if (ring && ring->open)
+			++open_rings;
+	}
+
+	if (open_rings == 0) {
+		pr_info("deallocating consumer %p\n", consumer->consumer_id);
+
+		if (remove_from_list) {
+			list_del_rcu(&consumer->node);
+			synchronize_rcu();
+		}
+
+		for_each_online_cpu(cpu) {
+			struct ppm_ring_buffer_context *ring = per_cpu_ptr(consumer->ring_buffers, cpu);
+
+			if (ring)
+				free_ring_buffer(ring);
+		}
+
+		free_percpu(consumer->ring_buffers);
+
+		vfree(consumer);
+	}
+}
+
 /*
  * user I/O functions
  */
 static int ppm_open(struct inode *inode, struct file *filp)
 {
 	int ret;
-	struct ppm_ring_buffer_context *ring;
-	int ring_no = iminor(filp->f_dentry->d_inode);
+	int in_list = false;
+	struct ppm_ring_buffer_context *ring = NULL;
+	int ring_no = iminor(filp->f_path.dentry->d_inode);
+	struct task_struct *consumer_id = current;
+	struct ppm_consumer_t *consumer = NULL;
 
-	mutex_lock(&g_open_mutex);
+	/*
+	 * Tricky: to identify a consumer, attach the thread id
+	 * to the newly open file descriptor
+	 */
+	filp->private_data = consumer_id;
 
-	ring = per_cpu(g_ring_buffers, ring_no);
+	mutex_lock(&g_consumer_mutex);
+
+	consumer = ppm_find_consumer(consumer_id);
+	if (!consumer) {
+		unsigned int cpu;
+		unsigned int num_consumers = 0;
+		struct ppm_consumer_t *el = NULL;
+
+		rcu_read_lock();
+		list_for_each_entry_rcu(el, &g_consumer_list, node) {
+			++num_consumers;
+		}
+		rcu_read_unlock();
+
+		if (num_consumers >= max_consumers) {
+			pr_err("maximum number of consumers reached\n");
+			ret = -EBUSY;
+			goto cleanup_open;
+		}
+
+		pr_info("adding new consumer %p\n", consumer_id);
+
+		consumer = vmalloc(sizeof(struct ppm_consumer_t));
+		if (!consumer) {
+			pr_err("can't allocate consumer\n");
+			ret = -ENOMEM;
+			goto cleanup_open;
+		}
+
+		consumer->consumer_id = consumer_id;
+
+		/*
+		 * Initialize the ring buffers array
+		 */
+		consumer->ring_buffers = alloc_percpu(struct ppm_ring_buffer_context);
+		if (consumer->ring_buffers == NULL) {
+			pr_err("can't allocate the ring buffer array\n");
+
+			vfree(consumer);
+
+			ret = -ENOMEM;
+			goto cleanup_open;
+		}
+
+		for_each_online_cpu(cpu) {
+			struct ppm_ring_buffer_context *ring = per_cpu_ptr(consumer->ring_buffers, cpu);
+
+			ring->str_storage = NULL;
+			ring->buffer = NULL;
+			ring->info = NULL;
+		}
+
+		for_each_online_cpu(cpu) {
+			pr_info("initializing ring buffer for CPU %u\n", cpu);
+
+			if (!init_ring_buffer(per_cpu_ptr(consumer->ring_buffers, cpu))) {
+				pr_err("can't initialize the ring buffer for CPU %u\n", cpu);
+				ret = -ENOMEM;
+				goto err_init_ring_buffer;
+			}
+		}
+
+		list_add_rcu(&consumer->node, &g_consumer_list);
+		in_list = true;
+	} else {
+		vpr_info("found already existent consumer %p\n", consumer_id);
+	}
+
+	ring = per_cpu_ptr(consumer->ring_buffers, ring_no);
 
 	if (ring->open) {
-		pr_err("invalid operation: attempting to open device %d multiple times\n", ring_no);
+		pr_err("invalid operation: attempting to open device %d multiple times for consumer %p\n", ring_no, consumer->consumer_id);
 		ret = -EBUSY;
 		goto cleanup_open;
 	}
 
-	vpr_info("opening ring %d\n", ring_no);
+	vpr_info("opening ring %d, consumer %p\n", ring_no, consumer->consumer_id);
 
 	/*
 	 * ring->preempt_count is not reset to 0 on purpose, to prevent a race condition:
-	 * if the same device is quickly closed and then reopened, record_event() might still be executing 
-	 * (with ring->preempt_count to 1) while ppm_open() resets ring->preempt_count to 0. 
-	 * When record_event() will exit, it will decrease 
+	 * if the same device is quickly closed and then reopened, record_event() might still be executing
+	 * (with ring->preempt_count to 1) while ppm_open() resets ring->preempt_count to 0.
+	 * When record_event() will exit, it will decrease
 	 * ring->preempt_count which will become < 0, leading to the complete loss of all the events for that CPU.
 	 */
-	g_dropping_mode = 0;
-	g_snaplen = RW_SNAPLEN;
-	g_sampling_ratio = 1;
-	g_sampling_interval = 0;
-	g_is_dropping = 0;
+	consumer->dropping_mode = 0;
+	consumer->snaplen = RW_SNAPLEN;
+	consumer->sampling_ratio = 1;
+	consumer->sampling_interval = 0;
+	consumer->is_dropping = 0;
+	consumer->do_dynamic_snaplen = false;
+	consumer->need_to_insert_drop_e = 0;
+	consumer->need_to_insert_drop_x = 0;
 	bitmap_fill(g_events_mask, PPM_EVENT_MAX); /* Enable all syscall to be passed to userspace */
 	ring->info->head = 0;
 	ring->info->tail = 0;
@@ -228,12 +343,12 @@ static int ppm_open(struct inode *inode, struct file *filp)
 	ring->info->n_drops_pf = 0;
 	ring->info->n_preemptions = 0;
 	ring->info->n_context_switches = 0;
-	ring->capture_enabled = true;
+	ring->capture_enabled = false;
 	getnstimeofday(&ring->last_print_time);
 	ring->open = true;
 
 	if (!g_tracepoint_registered) {
-		vpr_info("starting capture\n");
+		pr_info("starting capture\n");
 		/*
 		 * Enable the tracepoints
 		 */
@@ -265,9 +380,8 @@ static int ppm_open(struct inode *inode, struct file *filp)
 		g_tracepoint_registered = true;
 	}
 
-	++g_open_count;
 	ret = 0;
-	
+
 	goto cleanup_open;
 
 err_sched_switch:
@@ -278,8 +392,10 @@ err_sys_enter:
 	compat_unregister_trace(syscall_exit_probe, "sys_exit", tp_sys_exit);
 err_sys_exit:
 	ring->open = false;
+err_init_ring_buffer:
+	check_remove_consumer(consumer, in_list);
 cleanup_open:
-	mutex_unlock(&g_open_mutex);
+	mutex_unlock(&g_consumer_mutex);
 
 	return ret;
 }
@@ -288,35 +404,48 @@ static int ppm_release(struct inode *inode, struct file *filp)
 {
 	int ret;
 	struct ppm_ring_buffer_context *ring;
-	int ring_no = iminor(filp->f_dentry->d_inode);
+	int ring_no = iminor(filp->f_path.dentry->d_inode);
+	struct task_struct *consumer_id = filp->private_data;
+	struct ppm_consumer_t *consumer = NULL;
 
-	mutex_lock(&g_open_mutex);
+	mutex_lock(&g_consumer_mutex);
 
-	ring = per_cpu(g_ring_buffers, ring_no);
+	consumer = ppm_find_consumer(consumer_id);
+	if (!consumer) {
+		pr_err("release: unknown consumer %p\n", consumer_id);
+		ret = -EBUSY;
+		goto cleanup_release;
+	}
+
+	ring = per_cpu_ptr(consumer->ring_buffers, ring_no);
 
 	if (!ring->open) {
-		pr_err("attempting to close unopened device %d\n", ring_no);
+		pr_err("attempting to close unopened device %d for consumer %p\n", ring_no, consumer_id);
 		ret = -EBUSY;
 		goto cleanup_release;
 	}
 
 	ring->capture_enabled = false;
 
-	vpr_info("closing ring %d, evt:%llu, dr_buf:%llu, dr_pf:%llu, pr:%llu, cs:%llu\n",
+	vpr_info("closing ring %d, consumer:%p evt:%llu, dr_buf:%llu, dr_pf:%llu, pr:%llu, cs:%llu\n",
 	       ring_no,
+	       consumer_id,
 	       ring->info->n_evts,
 	       ring->info->n_drops_buffer,
 	       ring->info->n_drops_pf,
 	       ring->info->n_preemptions,
 	       ring->info->n_context_switches);
 
+	ring->open = false;
+
+	check_remove_consumer(consumer, true);
+
 	/*
 	 * The last closed device stops event collection
 	 */
-	--g_open_count;
-	if (g_open_count == 0) {
+	if (list_empty(&g_consumer_list)) {
 		if (g_tracepoint_registered) {
-			vpr_info("stopping capture\n");
+			pr_info("no more consumers, stopping capture\n");
 
 			compat_unregister_trace(syscall_exit_probe, "sys_exit", tp_sys_exit);
 			compat_unregister_trace(syscall_enter_probe, "sys_enter", tp_sys_enter);
@@ -332,58 +461,80 @@ static int ppm_release(struct inode *inode, struct file *filp)
 		}
 	}
 
-	ring->open = false;
 	ret = 0;
 
 cleanup_release:
-	mutex_unlock(&g_open_mutex);
+	mutex_unlock(&g_consumer_mutex);
 
 	return ret;
 }
 
 static long ppm_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
 {
+	int ret;
+	struct task_struct *consumer_id = filp->private_data;
+	struct ppm_consumer_t *consumer = NULL;
+
+	mutex_lock(&g_consumer_mutex);
+
+	consumer = ppm_find_consumer(consumer_id);
+	if (!consumer) {
+		pr_err("ioctl: unknown consumer %p\n", consumer_id);
+		ret = -EBUSY;
+		goto cleanup_ioctl;
+	}
+
 	switch (cmd) {
 	case PPM_IOCTL_DISABLE_CAPTURE:
 	{
-		int ring_no = iminor(filp->f_dentry->d_inode);
-		struct ppm_ring_buffer_context *ring = per_cpu(g_ring_buffers, ring_no);
+		int ring_no = iminor(filp->f_path.dentry->d_inode);
+		struct ppm_ring_buffer_context *ring = per_cpu_ptr(consumer->ring_buffers, ring_no);
 
-		mutex_lock(&g_open_mutex);
 		ring->capture_enabled = false;
-		mutex_unlock(&g_open_mutex);
 
-		vpr_info("PPM_IOCTL_DISABLE_CAPTURE for ring %d\n", ring_no);
+		vpr_info("PPM_IOCTL_DISABLE_CAPTURE for ring %d, consumer %p\n", ring_no, consumer_id);
 
-		return 0;
+		ret = 0;
+		goto cleanup_ioctl;
 	}
 	case PPM_IOCTL_ENABLE_CAPTURE:
 	{
-		int ring_no = iminor(filp->f_dentry->d_inode);
-		struct ppm_ring_buffer_context *ring = per_cpu(g_ring_buffers, ring_no);
+		int ring_no = iminor(filp->f_path.dentry->d_inode);
+		struct ppm_ring_buffer_context *ring = per_cpu_ptr(consumer->ring_buffers, ring_no);
 
-		mutex_lock(&g_open_mutex);
 		ring->capture_enabled = true;
-		mutex_unlock(&g_open_mutex);
 
-		vpr_info("PPM_IOCTL_ENABLE_CAPTURE for ring %d\n", ring_no);
+		vpr_info("PPM_IOCTL_ENABLE_CAPTURE for ring %d, consumer %p\n", ring_no, consumer_id);
 
-		return 0;
+		ret = 0;
+		goto cleanup_ioctl;
 	}
 	case PPM_IOCTL_DISABLE_DROPPING_MODE:
 	{
-		g_dropping_mode = 0;
-		vpr_info("PPM_IOCTL_DISABLE_DROPPING_MODE\n");
-		g_sampling_interval = 1000000000;
-		g_sampling_ratio = 1;
-		return 0;
+		struct timespec ts;
+
+		vpr_info("PPM_IOCTL_DISABLE_DROPPING_MODE, consumer %p\n", consumer_id);
+
+		consumer->dropping_mode = 0;
+		consumer->sampling_interval = 1000000000;
+		consumer->sampling_ratio = 1;
+
+		/*
+		 * Push an event into the ring buffer so that the user can know that dropping
+		 * mode has been disabled
+		 */
+		getnstimeofday(&ts);
+		record_event_consumer(consumer, PPME_SYSDIGEVENT_E, NULL, -1, UF_NEVER_DROP, (void *)DEI_DISABLE_DROPPING, (void *)0, &ts);
+
+		ret = 0;
+		goto cleanup_ioctl;
 	}
 	case PPM_IOCTL_ENABLE_DROPPING_MODE:
 	{
 		u32 new_sampling_ratio;
 
-		g_dropping_mode = 1;
-		vpr_info("PPM_IOCTL_ENABLE_DROPPING_MODE\n");
+		consumer->dropping_mode = 1;
+		vpr_info("PPM_IOCTL_ENABLE_DROPPING_MODE, consumer %p\n", consumer_id);
 
 		new_sampling_ratio = (u32)arg;
 
@@ -399,17 +550,19 @@ static long ppm_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
 			return -EINVAL;
 		}
 
-		g_sampling_interval = 1000000000 / new_sampling_ratio;
-		g_sampling_ratio = new_sampling_ratio;
+		consumer->sampling_interval = 1000000000 / new_sampling_ratio;
+		consumer->sampling_ratio = new_sampling_ratio;
 
 		vpr_info("new sampling ratio: %d\n", new_sampling_ratio);
-		return 0;
+
+		ret = 0;
+		goto cleanup_ioctl;
 	}
 	case PPM_IOCTL_SET_SNAPLEN:
 	{
 		u32 new_snaplen;
 
-		vpr_info("PPM_IOCTL_SET_SNAPLEN\n");
+		vpr_info("PPM_IOCTL_SET_SNAPLEN, consumer %p\n", consumer_id);
 		new_snaplen = (u32)arg;
 
 		if (new_snaplen > RW_MAX_SNAPLEN) {
@@ -417,29 +570,31 @@ static long ppm_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
 			return -EINVAL;
 		}
 
-		g_snaplen = new_snaplen;
+		consumer->snaplen = new_snaplen;
 
-		vpr_info("new snaplen: %d\n", g_snaplen);
-		return 0;
+		vpr_info("new snaplen: %d\n", consumer->snaplen);
+
+		ret = 0;
+		goto cleanup_ioctl;
 	}
-
 	case PPM_IOCTL_MASK_ZERO_EVENTS:
 	{
-		vpr_info("PPM_IOCTL_MASK_ZERO_EVENTS\n");
+		vpr_info("PPM_IOCTL_MASK_ZERO_EVENTS, consumer %p\n", consumer_id);
 
 		bitmap_zero(g_events_mask, PPM_EVENT_MAX);
 
 		/* Used for dropping events so they must stay on */
 		set_bit(PPME_DROP_E, g_events_mask);
 		set_bit(PPME_DROP_X, g_events_mask);
-		return 0;
-	}
 
+		ret = 0;
+		goto cleanup_ioctl;
+	}
 	case PPM_IOCTL_MASK_SET_EVENT:
 	{
 		u32 syscall_to_set = (u32)arg;
 
-		vpr_info("PPM_IOCTL_MASK_SET_EVENT (%u)\n", syscall_to_set);
+		vpr_info("PPM_IOCTL_MASK_SET_EVENT (%u), consumer %p\n", syscall_to_set, consumer_id);
 
 		if (syscall_to_set > PPM_EVENT_MAX) {
 			pr_err("invalid syscall %u\n", syscall_to_set);
@@ -447,14 +602,15 @@ static long ppm_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
 		}
 
 		set_bit(syscall_to_set, g_events_mask);
-		return 0;
-	}
 
+		ret = 0;
+		goto cleanup_ioctl;
+	}
 	case PPM_IOCTL_MASK_UNSET_EVENT:
 	{
 		u32 syscall_to_unset = (u32)arg;
 
-		vpr_info("PPM_IOCTL_MASK_UNSET_EVENT (%u)\n", syscall_to_unset);
+		vpr_info("PPM_IOCTL_MASK_UNSET_EVENT (%u), consumer %p\n", syscall_to_unset, consumer_id);
 
 		if (syscall_to_unset > NR_syscalls) {
 			pr_err("invalid syscall %u\n", syscall_to_unset);
@@ -462,27 +618,106 @@ static long ppm_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
 		}
 
 		clear_bit(syscall_to_unset, g_events_mask);
-		return 0;
+
+		ret = 0;
+		goto cleanup_ioctl;
+	}
+	case PPM_IOCTL_DISABLE_DYNAMIC_SNAPLEN:
+	{
+		consumer->do_dynamic_snaplen = false;
+
+		ret = 0;
+		goto cleanup_ioctl;
+	}
+	case PPM_IOCTL_ENABLE_DYNAMIC_SNAPLEN:
+	{
+		consumer->do_dynamic_snaplen = true;
+
+		ret = 0;
+		goto cleanup_ioctl;
+	}
+	case PPM_IOCTL_GET_VTID:
+	case PPM_IOCTL_GET_VPID:
+	{
+		pid_t vid;
+		struct pid *pid;
+		struct task_struct *task;
+		struct pid_namespace *ns;
+
+		rcu_read_lock();
+		pid = find_pid_ns(arg, &init_pid_ns);
+		if(!pid) {
+			rcu_read_unlock();
+			ret = -EINVAL;
+			goto cleanup_ioctl;
+		}
+
+		task = pid_task(pid, PIDTYPE_PID);
+		if (!task) {
+			rcu_read_unlock();
+			ret = -EINVAL;
+			goto cleanup_ioctl;
+		}
+
+		ns = ns_of_pid(pid);
+		if(!pid) {
+			rcu_read_unlock();
+			ret = -EINVAL;
+			goto cleanup_ioctl;
+		}
+
+		if(cmd == PPM_IOCTL_GET_VTID)
+			vid = task_pid_nr_ns(task, ns);
+		else
+			vid = task_tgid_nr_ns(task, ns);
+
+		rcu_read_unlock();
+		ret = vid;
+		goto cleanup_ioctl;
+	}
+	case PPM_IOCTL_GET_CURRENT_TID:
+		ret = task_pid_nr(current);
+		goto cleanup_ioctl;
+	case PPM_IOCTL_GET_CURRENT_PID:
+		ret = task_tgid_nr(current);
+		goto cleanup_ioctl;
+	default:
+		ret = -ENOTTY;
+		goto cleanup_ioctl;
 	}
 
-	default:
-		return -ENOTTY;
-	}
+cleanup_ioctl:
+	mutex_unlock(&g_consumer_mutex);
+
+	return ret;
 }
 
 static int ppm_mmap(struct file *filp, struct vm_area_struct *vma)
 {
+	int ret;
+	struct task_struct *consumer_id = filp->private_data;
+	struct ppm_consumer_t *consumer = NULL;
+
+	mutex_lock(&g_consumer_mutex);
+
+	consumer = ppm_find_consumer(consumer_id);
+	if (!consumer) {
+		pr_err("mmap: unknown consumer %p\n", consumer_id);
+		ret = -EIO;
+		goto cleanup_mmap;
+	}
+
 	if (vma->vm_pgoff == 0) {
-		int ret;
 		long length = vma->vm_end - vma->vm_start;
 		unsigned long useraddr = vma->vm_start;
 		unsigned long pfn;
 		char *vmalloc_area_ptr;
 		char *orig_vmalloc_area_ptr;
-		int ring_no = iminor(filp->f_dentry->d_inode);
+		int ring_no = iminor(filp->f_path.dentry->d_inode);
 		struct ppm_ring_buffer_context *ring;
 
-		vpr_info("mmap for CPU %d, start=%lu len=%ld page_size=%lu\n",
+		vpr_info("mmap for consumer %p, CPU %d, start=%lu len=%ld page_size=%lu\n",
+			   consumer_id,
 		       ring_no,
 		       useraddr,
 		       length,
@@ -495,18 +730,20 @@ static int ppm_mmap(struct file *filp, struct vm_area_struct *vma)
 			pr_err("Ring buffer size too small (%ld bytes, must be at least %ld bytes\n",
 			       (long)RING_BUF_SIZE,
 			       (long)PAGE_SIZE);
-			return -EIO;
+			ret = -EIO;
+			goto cleanup_mmap;
 		}
 
 		if (RING_BUF_SIZE / PAGE_SIZE * PAGE_SIZE != RING_BUF_SIZE) {
 			pr_err("Ring buffer size is not a multiple of the page size\n");
-			return -EIO;
+			ret = -EIO;
+			goto cleanup_mmap;
 		}
 
 		/*
 		 * Retrieve the ring structure for this CPU
 		 */
-		ring = per_cpu(g_ring_buffers, ring_no);
+		ring = per_cpu_ptr(consumer->ring_buffers, ring_no);
 
 		if (length <= PAGE_SIZE) {
 			/*
@@ -524,10 +761,11 @@ static int ppm_mmap(struct file *filp, struct vm_area_struct *vma)
 					      PAGE_SIZE, PAGE_SHARED);
 			if (ret < 0) {
 				pr_err("remap_pfn_range failed (1)\n");
-				return ret;
+				goto cleanup_mmap;
 			}
 
-			return 0;
+			ret = 0;
+			goto cleanup_mmap;
 		} else if (length == RING_BUF_SIZE * 2) {
 			long mlength;
 
@@ -545,7 +783,8 @@ static int ppm_mmap(struct file *filp, struct vm_area_struct *vma)
 			 */
 			if (vma->vm_flags & VM_WRITE) {
 				pr_err("invalid mmap flags 0x%lx\n", vma->vm_flags);
-				return -EIO;
+				ret = -EIO;
+				goto cleanup_mmap;
 			}
 
 			/*
@@ -560,7 +799,7 @@ static int ppm_mmap(struct file *filp, struct vm_area_struct *vma)
 						      PAGE_SIZE, PAGE_SHARED);
 				if (ret < 0) {
 					pr_err("remap_pfn_range failed (1)\n");
-					return ret;
+					goto cleanup_mmap;
 				}
 
 				useraddr += PAGE_SIZE;
@@ -582,7 +821,7 @@ static int ppm_mmap(struct file *filp, struct vm_area_struct *vma)
 						      PAGE_SIZE, PAGE_SHARED);
 				if (ret < 0) {
 					pr_err("remap_pfn_range failed (1)\n");
-					return ret;
+					goto cleanup_mmap;
 				}
 
 				useraddr += PAGE_SIZE;
@@ -590,15 +829,22 @@ static int ppm_mmap(struct file *filp, struct vm_area_struct *vma)
 				mlength -= PAGE_SIZE;
 			}
 
-			return 0;
-		} else {
-			pr_err("Invalid mmap size %ld\n", length);
-			return -EIO;
+			ret = 0;
+			goto cleanup_mmap;
 		}
+
+		pr_err("Invalid mmap size %ld\n", length);
+		ret = -EIO;
+		goto cleanup_mmap;
 	}
 
 	pr_err("invalid pgoff %lu, must be 0\n", vma->vm_pgoff);
-	return -EIO;
+	ret = -EIO;
+
+cleanup_mmap:
+	mutex_unlock(&g_consumer_mutex);
+
+	return ret;
 }
 
 static ssize_t ppe_write(struct file *filp, const char __user *buf, size_t count, loff_t *f_pos)
@@ -692,41 +938,98 @@ static enum ppm_event_type parse_socketcall(struct event_filler_arguments *fille
 }
 #endif /* __NR_socketcall */
 
-static inline int drop_event(enum ppm_event_type event_type, int never_drop, struct timespec *ts)
+static inline void record_drop_e(struct ppm_consumer_t *consumer, struct timespec *ts)
 {
-	if (never_drop)
-		return 0;
+	if (record_event_consumer(consumer, PPME_DROP_E, NULL, -1, UF_NEVER_DROP, NULL, NULL, ts) == 0) {
+		consumer->need_to_insert_drop_e = 1;
+	} else {
+		if (consumer->need_to_insert_drop_e == 1)
+			pr_err("drop enter event delayed insert\n");
 
-	if (g_dropping_mode) {
-		if (ts->tv_nsec >= g_sampling_interval) {
-			if (g_is_dropping == 0) {
-				g_is_dropping = 1;
-				record_event(PPME_DROP_E, NULL, -1, 1, NULL, NULL);
+		consumer->need_to_insert_drop_e = 0;
+	}
+}
+
+static inline void record_drop_x(struct ppm_consumer_t *consumer, struct timespec *ts)
+{
+	if (record_event_consumer(consumer, PPME_DROP_X, NULL, -1, UF_NEVER_DROP, NULL, NULL, ts) == 0) {
+		consumer->need_to_insert_drop_x = 1;
+	} else {
+		if (consumer->need_to_insert_drop_x == 1)
+			pr_err("drop exit event delayed insert\n");
+
+		consumer->need_to_insert_drop_x = 0;
+	}
+}
+
+static inline int drop_event(struct ppm_consumer_t *consumer, enum ppm_event_type event_type, enum syscall_flags drop_flags, struct timespec *ts)
+{
+	if (drop_flags & UF_NEVER_DROP) {
+		ASSERT((drop_flags & UF_ALWAYS_DROP) == 0);
+		return 0;
+	}
+
+	if (consumer->dropping_mode) {
+		if (drop_flags & UF_ALWAYS_DROP) {
+			ASSERT((drop_flags & UF_NEVER_DROP) == 0);
+			return 1;
+		}
+
+		if (ts->tv_nsec >= consumer->sampling_interval) {
+			if (consumer->is_dropping == 0) {
+				consumer->is_dropping = 1;
+				record_drop_e(consumer, ts);
 			}
 
 			return 1;
-		} else {
-			if (g_is_dropping == 1) {
-				g_is_dropping = 0;
-				record_event(PPME_DROP_X, NULL, -1, 1, NULL, NULL);
-			}
+		}
+
+		if (consumer->is_dropping == 1) {
+			consumer->is_dropping = 0;
+			record_drop_x(consumer, ts);
 		}
 	}
 
 	return 0;
 }
 
-static void record_event(enum ppm_event_type event_type,
+static void record_event_all_consumers(enum ppm_event_type event_type,
 	struct pt_regs *regs,
 	long id,
-	int never_drop,
+	enum syscall_flags drop_flags,
 	struct task_struct *sched_prev,
 	struct task_struct *sched_next)
 {
+	struct ppm_consumer_t *consumer;
+	struct timespec ts;
+
+	getnstimeofday(&ts);
+
+	rcu_read_lock();
+	list_for_each_entry_rcu(consumer, &g_consumer_list, node) {
+		record_event_consumer(consumer, event_type, regs, id, drop_flags, sched_prev, sched_next, &ts);
+	}
+	rcu_read_unlock();
+}
+
+/*
+ * Returns 0 if the event is dropped
+ */
+static int record_event_consumer(struct ppm_consumer_t *consumer,
+	enum ppm_event_type event_type,
+	struct pt_regs *regs,
+	long id,
+	enum syscall_flags drop_flags,
+	struct task_struct *sched_prev,
+	struct task_struct *sched_next,
+	struct timespec *ts)
+{
+	int res = 0;
 	size_t event_size;
 	int next;
 	u32 freespace;
 	u32 usedspace;
+	u32 delta_from_end;
 	struct event_filler_arguments args;
 	u32 ttail;
 	u32 head;
@@ -734,33 +1037,41 @@ static void record_event(enum ppm_event_type event_type,
 	struct ppm_ring_buffer_info *ring_info;
 	int drop = 1;
 	int32_t cbres = PPM_SUCCESS;
-	struct timespec ts;
-
-	getnstimeofday(&ts);
+	int cpu;
 
 	if (!test_bit(event_type, g_events_mask))
-		return;
+		return res;
 
-	if (drop_event(event_type, never_drop, &ts))
-		return;
+	if (event_type != PPME_DROP_E && event_type != PPME_DROP_X) {
+		if (consumer->need_to_insert_drop_e == 1)
+			record_drop_e(consumer, ts);
+		else if (consumer->need_to_insert_drop_x == 1)
+			record_drop_x(consumer, ts);
+
+		if (drop_event(consumer, event_type, drop_flags, ts))
+			return res;
+	}
 
 	/*
 	 * FROM THIS MOMENT ON, WE HAVE TO BE SUPER FAST
 	 */
-	ring = get_cpu_var(g_ring_buffers);
+	cpu = get_cpu();
+	ring = per_cpu_ptr(consumer->ring_buffers, cpu);
 	ring_info = ring->info;
 
 	if (!ring->capture_enabled) {
-		put_cpu_var(g_ring_buffers);
-		return;
+		put_cpu();
+		return res;
 	}
 
 	ring_info->n_evts++;
 	if (sched_prev != NULL) {
-		ASSERT(sched_prev != NULL);
-		ASSERT(sched_next != NULL);
-		ASSERT(regs == NULL);
-		ring_info->n_context_switches++;
+		if (event_type != PPME_SYSDIGEVENT_E) {
+			ASSERT(sched_prev != NULL);
+			ASSERT(sched_next != NULL);
+			ASSERT(regs == NULL);
+			ring_info->n_context_switches++;
+		}
 	}
 
 	/*
@@ -768,10 +1079,10 @@ static void record_event(enum ppm_event_type event_type,
 	 */
 	if (unlikely(atomic_inc_return(&ring->preempt_count) != 1)) {
 		atomic_dec(&ring->preempt_count);
-		put_cpu_var(g_ring_buffers);
 		ring_info->n_preemptions++;
+		put_cpu();
 		ASSERT(false);
-		return;
+		return res;
 	}
 
 	/*
@@ -786,12 +1097,14 @@ static void record_event(enum ppm_event_type event_type,
 		freespace = RING_BUF_SIZE + ttail - head - 1;
 
 	usedspace = RING_BUF_SIZE - freespace - 1;
+	delta_from_end = RING_BUF_SIZE + (2 * PAGE_SIZE) - head - 1;
 
 	ASSERT(freespace <= RING_BUF_SIZE);
 	ASSERT(usedspace <= RING_BUF_SIZE);
 	ASSERT(ttail <= RING_BUF_SIZE);
 	ASSERT(head <= RING_BUF_SIZE);
-
+	ASSERT(delta_from_end < RING_BUF_SIZE + (2 * PAGE_SIZE));
+	ASSERT(delta_from_end > (2 * PAGE_SIZE) - 1);
 #ifdef __NR_socketcall
 	/*
 	 * If this is a socketcall system call, determine the correct event type
@@ -836,18 +1149,19 @@ static void record_event(enum ppm_event_type event_type,
 #ifdef PPM_ENABLE_SENTINEL
 		hdr->sentinel_begin = ring->nevents;
 #endif
-		hdr->ts = timespec_to_ns(&ts);
+		hdr->ts = timespec_to_ns(ts);
 		hdr->tid = current->pid;
 		hdr->type = event_type;
 
 		/*
 		 * Populate the parameters for the filler callback
 		 */
+		args.consumer = consumer;
 		args.buffer = ring->buffer + head + sizeof(struct ppm_evt_hdr);
 #ifdef PPM_ENABLE_SENTINEL
 		args.sentinel = ring->nevents;
 #endif
-		args.buffer_size = min(freespace, (u32)(2 * PAGE_SIZE)) - sizeof(struct ppm_evt_hdr); /* freespace is guaranteed to be bigger than sizeof(struct ppm_evt_hdr) */
+		args.buffer_size = min(freespace, delta_from_end) - sizeof(struct ppm_evt_hdr); /* freespace is guaranteed to be bigger than sizeof(struct ppm_evt_hdr) */
 		args.event_type = event_type;
 		args.regs = regs;
 		args.sched_prev = sched_prev;
@@ -857,6 +1171,7 @@ static void record_event(enum ppm_event_type event_type,
 		args.arg_data_size = args.buffer_size - args.arg_data_offset;
 		args.nevents = ring->nevents;
 		args.str_storage = ring->str_storage;
+		args.enforce_snaplen = false;
 
 		/*
 		 * Fire the filler callback
@@ -878,6 +1193,9 @@ static void record_event(enum ppm_event_type event_type,
 			 * Validate that the filler added the right number of parameters
 			 */
 			if (likely(args.curarg == args.nargs)) {
+				/*
+				 * The event was successfully insterted in the buffer
+				 */
 				event_size = sizeof(struct ppm_evt_hdr) + args.arg_data_offset;
 				hdr->len = event_size;
 				drop = 0;
@@ -892,6 +1210,8 @@ static void record_event(enum ppm_event_type event_type,
 	}
 
 	if (likely(!drop)) {
+		res = 1;
+
 		next = head + event_size;
 
 		if (unlikely(next >= RING_BUF_SIZE)) {
@@ -937,8 +1257,9 @@ static void record_event(enum ppm_event_type event_type,
 	}
 
 #ifdef _DEBUG
-	if (ts.tv_sec > ring->last_print_time.tv_sec + 1) {
-		vpr_info("CPU%d, use:%d%%, ev:%llu, dr_buf:%llu, dr_pf:%llu, pr:%llu, cs:%llu\n",
+	if (ts->tv_sec > ring->last_print_time.tv_sec + 1) {
+		vpr_info("consumer:%p CPU:%d, use:%d%%, ev:%llu, dr_buf:%llu, dr_pf:%llu, pr:%llu, cs:%llu\n",
+			   consumer->consumer_id,
 		       smp_processor_id(),
 		       (usedspace * 100) / RING_BUF_SIZE,
 		       ring_info->n_evts,
@@ -947,14 +1268,14 @@ static void record_event(enum ppm_event_type event_type,
 		       ring_info->n_preemptions,
 		       ring->info->n_context_switches);
 
-		ring->last_print_time = ts;
+		ring->last_print_time = *ts;
 	}
 #endif
 
 	atomic_dec(&ring->preempt_count);
-	put_cpu_var(g_ring_buffers);
+	put_cpu();
 
-	return;
+	return res;
 }
 
 TRACEPOINT_PROBE(syscall_enter_probe, struct pt_regs *regs, long id)
@@ -974,12 +1295,24 @@ TRACEPOINT_PROBE(syscall_enter_probe, struct pt_regs *regs, long id)
 	table_index = id - SYSCALL_TABLE_ID0;
 	if (likely(table_index >= 0 && table_index < SYSCALL_TABLE_SIZE)) {
 		int used = g_syscall_table[table_index].flags & UF_USED;
-		int never_drop = g_syscall_table[table_index].flags & UF_NEVER_DROP;
+		enum syscall_flags drop_flags = g_syscall_table[table_index].flags;
+		enum ppm_event_type type;
+
+#ifdef __NR_socketcall
+		if (id == __NR_socketcall) {
+			used = true;
+			drop_flags = UF_NEVER_DROP;
+			type = PPME_GENERIC_E;
+		} else
+			type = g_syscall_table[table_index].enter_event_type;
+#else
+		type = g_syscall_table[table_index].enter_event_type;
+#endif
 
 		if (used)
-			record_event(g_syscall_table[table_index].enter_event_type, regs, id, never_drop, NULL, NULL);
+			record_event_all_consumers(type, regs, id, drop_flags, NULL, NULL);
 		else
-			record_event(PPME_GENERIC_E, regs, id, false, NULL, NULL);
+			record_event_all_consumers(PPME_GENERIC_E, regs, id, UF_ALWAYS_DROP, NULL, NULL);
 	}
 }
 
@@ -1003,12 +1336,24 @@ TRACEPOINT_PROBE(syscall_exit_probe, struct pt_regs *regs, long ret)
 	table_index = id - SYSCALL_TABLE_ID0;
 	if (likely(table_index >= 0 && table_index < SYSCALL_TABLE_SIZE)) {
 		int used = g_syscall_table[table_index].flags & UF_USED;
-		int never_drop = g_syscall_table[table_index].flags & UF_NEVER_DROP;
+		enum syscall_flags drop_flags = g_syscall_table[table_index].flags;
+		enum ppm_event_type type;
+
+#ifdef __NR_socketcall
+		if (id == __NR_socketcall) {
+			used = true;
+			drop_flags = UF_NEVER_DROP;
+			type = PPME_GENERIC_X;
+		} else
+			type = g_syscall_table[table_index].exit_event_type;
+#else
+		type = g_syscall_table[table_index].exit_event_type;
+#endif
 
 		if (used)
-			record_event(g_syscall_table[table_index].exit_event_type, regs, id, never_drop, NULL, NULL);
+			record_event_all_consumers(type, regs, id, drop_flags, NULL, NULL);
 		else
-			record_event(PPME_GENERIC_X, regs, id, false, NULL, NULL);
+			record_event_all_consumers(PPME_GENERIC_X, regs, id, UF_ALWAYS_DROP, NULL, NULL);
 	}
 }
 
@@ -1024,7 +1369,7 @@ TRACEPOINT_PROBE(syscall_procexit_probe, struct task_struct *p)
 		return;
 	}
 
-	record_event(PPME_PROCEXIT_E, NULL, -1, 1, NULL, NULL);
+	record_event_all_consumers(PPME_PROCEXIT_1_E, NULL, -1, UF_NEVER_DROP, p, p);
 }
 
 #include <linux/ip.h>
@@ -1038,33 +1383,24 @@ TRACEPOINT_PROBE(sched_switch_probe, struct rq *rq, struct task_struct *prev, st
 TRACEPOINT_PROBE(sched_switch_probe, struct task_struct *prev, struct task_struct *next)
 #endif
 {
-	record_event(PPME_SCHEDSWITCH_6_E,
+	record_event_all_consumers(PPME_SCHEDSWITCH_6_E,
 		NULL,
 		-1,
-		0,
+		UF_USED,
 		prev,
 		next);
 }
 #endif
 
-static struct ppm_ring_buffer_context *alloc_ring_buffer(struct ppm_ring_buffer_context **ring)
+static int init_ring_buffer(struct ppm_ring_buffer_context *ring)
 {
 	unsigned int j;
 
-	/*
-	 * Allocate the ring descriptor
-	 */
-	*ring = vmalloc(sizeof(struct ppm_ring_buffer_context));
-	if (*ring == NULL) {
-		pr_err("Error allocating ring memory\n");
-		return NULL;
-	}
-
 	/*
 	 * Allocate the string storage in the ring descriptor
 	 */
-	(*ring)->str_storage = (char *)__get_free_page(GFP_USER);
-	if (!(*ring)->str_storage) {
+	ring->str_storage = (char *)__get_free_page(GFP_USER);
+	if (!ring->str_storage) {
 		pr_err("Error allocating the string storage\n");
 		goto err_str_storage;
 	}
@@ -1074,20 +1410,20 @@ static struct ppm_ring_buffer_context *alloc_ring_buffer(struct ppm_ring_buffer_
 	 * Note how we allocate 2 additional pages: they are used as additional overflow space for
 	 * the event data generation functions, so that they always operate on a contiguous buffer.
 	 */
-	(*ring)->buffer = vmalloc(RING_BUF_SIZE + 2 * PAGE_SIZE);
-	if ((*ring)->buffer == NULL) {
+	ring->buffer = vmalloc(RING_BUF_SIZE + 2 * PAGE_SIZE);
+	if (ring->buffer == NULL) {
 		pr_err("Error allocating ring memory\n");
 		goto err_buffer;
 	}
 
 	for (j = 0; j < RING_BUF_SIZE + 2 * PAGE_SIZE; j++)
-		(*ring)->buffer[j] = 0;
+		ring->buffer[j] = 0;
 
 	/*
 	 * Allocate the buffer info structure
 	 */
-	(*ring)->info = vmalloc(sizeof(struct ppm_ring_buffer_info));
-	if ((*ring)->info == NULL) {
+	ring->info = vmalloc(sizeof(struct ppm_ring_buffer_info));
+	if (ring->info == NULL) {
 		pr_err("Error allocating ring memory\n");
 		goto err_ring_info;
 	}
@@ -1095,39 +1431,43 @@ static struct ppm_ring_buffer_context *alloc_ring_buffer(struct ppm_ring_buffer_
 	/*
 	 * Initialize the buffer info structure
 	 */
-	(*ring)->open = false;
-	(*ring)->capture_enabled = false;
-	(*ring)->info->head = 0;
-	(*ring)->info->tail = 0;
-	(*ring)->nevents = 0;
-	(*ring)->info->n_evts = 0;
-	(*ring)->info->n_drops_buffer = 0;
-	(*ring)->info->n_drops_pf = 0;
-	(*ring)->info->n_preemptions = 0;
-	(*ring)->info->n_context_switches = 0;
-	atomic_set(&(*ring)->preempt_count, 0);
-	getnstimeofday(&(*ring)->last_print_time);
+	ring->open = false;
+	ring->capture_enabled = false;
+	ring->info->head = 0;
+	ring->info->tail = 0;
+	ring->nevents = 0;
+	ring->info->n_evts = 0;
+	ring->info->n_drops_buffer = 0;
+	ring->info->n_drops_pf = 0;
+	ring->info->n_preemptions = 0;
+	ring->info->n_context_switches = 0;
+	atomic_set(&ring->preempt_count, 0);
+	getnstimeofday(&ring->last_print_time);
 
 	pr_info("CPU buffer initialized, size=%d\n", RING_BUF_SIZE);
 
-	return *ring;
+	return 1;
 
 err_ring_info:
-	vfree((void *)(*ring)->buffer);
+	vfree((void *)ring->buffer);
+	ring->buffer = NULL;
 err_buffer:
-	free_page((unsigned long)(*ring)->str_storage);
+	free_page((unsigned long)ring->str_storage);
+	ring->str_storage = NULL;
 err_str_storage:
-	vfree(*ring);
-
-	return NULL;
+	return 0;
 }
 
 static void free_ring_buffer(struct ppm_ring_buffer_context *ring)
 {
-	vfree(ring->info);
-	vfree((void *)ring->buffer);
-	free_page((unsigned long)ring->str_storage);
-	vfree(ring);
+	if (ring->info)
+		vfree(ring->info);
+
+	if (ring->buffer)
+		vfree((void *)ring->buffer);
+
+	if (ring->str_storage)
+		free_page((unsigned long)ring->str_storage);
 }
 
 #if (LINUX_VERSION_CODE >= KERNEL_VERSION(3, 15, 0))
@@ -1211,26 +1551,11 @@ int sysdig_init(void)
 	if (ret < 0)
 		goto init_module_err;
 
-	/*
-	 * Initialize the ring buffers array
-	 */
 	num_cpus = 0;
 	for_each_online_cpu(cpu) {
-		per_cpu(g_ring_buffers, cpu) = NULL;
 		++num_cpus;
 	}
 
-	for_each_online_cpu(cpu) {
-		pr_info("initializing ring buffer for CPU %u\n", cpu);
-
-		alloc_ring_buffer(&per_cpu(g_ring_buffers, cpu));
-		if (per_cpu(g_ring_buffers, cpu) == NULL) {
-			pr_err("can't initialize the ring buffer for CPU %u\n", cpu);
-			ret = -ENOMEM;
-			goto init_module_err;
-		}
-	}
-
 	/*
 	 * Initialize the user I/O
 	 * ( + 1 for sysdig-events)
@@ -1318,22 +1643,23 @@ int sysdig_init(void)
 		goto init_module_err;
 	}
 
+	/*
+	 * Snaplen lookahead initialization
+	 */
+	if (dpi_lookahead_init() != PPM_SUCCESS) {
+		pr_err("initializing lookahead-based snaplen  %s\n", PPE_DEVICE_NAME);
+		ret = -EFAULT;
+		goto init_module_err;
+	}
+
 	/*
 	 * All ok. Final initalizations.
 	 */
-	g_open_count = 0;
 	g_tracepoint_registered = false;
-	g_dropping_mode = 0;
 
 	return 0;
 
 init_module_err:
-	for_each_online_cpu(cpu)
-		if (per_cpu(g_ring_buffers, cpu) != NULL)
-			free_ring_buffer(per_cpu(g_ring_buffers, cpu));
-
-	/* remove_proc_entry(PPM_DEVICE_NAME, NULL); */
-
 	if (g_ppe_dev != NULL)
 		device_destroy(g_ppm_class, MKDEV(g_ppm_major, g_ppm_numdevs));
 
@@ -1359,15 +1685,9 @@ init_module_err:
 void sysdig_exit(void)
 {
 	int j;
-	int cpu;
 
 	pr_info("driver unloading\n");
 
-	/* remove_proc_entry(PPM_DEVICE_NAME, NULL); */
-
-	for_each_online_cpu(cpu)
-		free_ring_buffer(per_cpu(g_ring_buffers, cpu));
-
 	for (j = 0; j < g_ppm_numdevs; ++j) {
 		device_destroy(g_ppm_class, g_ppm_devs[j].dev);
 		cdev_del(&g_ppm_devs[j].cdev);
@@ -1392,4 +1712,7 @@ void sysdig_exit(void)
 
 module_init(sysdig_init);
 module_exit(sysdig_exit);
-module_param(verbose, bool, 0);
+module_param(max_consumers, uint, 0444);
+MODULE_PARM_DESC(max_consumers, "Maximum number of consumers that can simultaneously open the devices");
+module_param(verbose, bool, 0444);
+MODULE_PARM_DESC(verbose, "Enable verbose logging");
