@@ -150,7 +150,6 @@ int32_t scap_fd_info_to_string(scap_fdinfo *fdi, OUT char *str, uint32_t stlen)
  		snprintf(str, stlen, "<UNSUPPORTED>");
  		break;
 	default:
-		printf("type = %d\n", fdi->type);
 		ASSERT(false);
 		return SCAP_FAILURE;
 	}
@@ -448,6 +447,23 @@ uint32_t scap_fd_read_from_disk(scap_t *handle, OUT scap_fdinfo *fdi, OUT size_t
 	return res;
 }
 
+void scap_fd_free_ns_sockets_list(scap_t *handle, struct scap_ns_socket_list **sockets)
+{
+	struct scap_ns_socket_list *fdi;
+	struct scap_ns_socket_list *tfdi;
+
+	if(*sockets)
+	{
+		HASH_ITER(hh, *sockets, fdi, tfdi)
+		{
+			HASH_DEL(*sockets, fdi);
+			scap_fd_free_table(handle, &fdi->sockets);
+			free(fdi);
+		}
+		*sockets = NULL;
+	}
+}
+
 void scap_fd_free_table(scap_t *handle, scap_fdinfo **fds)
 {
 	struct scap_fdinfo *fdi;
@@ -524,20 +540,23 @@ int32_t scap_add_fd_to_proc_table(scap_t *handle, scap_threadinfo *tinfo, scap_f
 		//
 		HASH_DEL(tinfo->fdlist, tfdi);
 		free(tfdi);
-
-		//      ASSERT(false);
-		//      snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "duplicate fd %"PRIu64"for process%"PRIu64, fdi->fd, tinfo->tid);
-		//      return SCAP_FAILURE;
 	}
 
 	//
-	// Add the fd to the table
+	// Add the fd to the table, or fire the notification callback
 	//
-	HASH_ADD_INT64(tinfo->fdlist, fd, fdi);
-	if(uth_status != SCAP_SUCCESS)
+	if(handle->m_proc_callback == NULL)
 	{
-		snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "process table allocation error (2)");
-		return SCAP_FAILURE;
+		HASH_ADD_INT64(tinfo->fdlist, fd, fdi);
+		if(uth_status != SCAP_SUCCESS)
+		{
+			snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "process table allocation error (2)");
+			return SCAP_FAILURE;
+		}
+	}
+	else
+	{
+		handle->m_proc_callback(handle->m_proc_callback_context, tinfo->tid, tinfo, fdi, handle);
 	}
 
 	return SCAP_SUCCESS;
@@ -578,7 +597,6 @@ int32_t scap_fd_handle_regular_file(scap_t *handle, char *fname, scap_threadinfo
 {
 	char link_name[1024];
 	ssize_t r;
-	int32_t res;
 
 	r = readlink(fname, link_name, 1024);
 	if (r <= 0)
@@ -623,24 +641,48 @@ int32_t scap_fd_handle_regular_file(scap_t *handle, char *fname, scap_threadinfo
 		strncpy(fdi->info.fname, link_name, SCAP_MAX_PATH_SIZE);
 	}
 
-	res = scap_add_fd_to_proc_table(handle, tinfo, fdi);
-	return res;
+	return scap_add_fd_to_proc_table(handle, tinfo, fdi);
 }
 
-int32_t scap_fd_handle_socket(scap_t *handle, char *fname, scap_threadinfo *tinfo, scap_fdinfo *fdi, scap_fdinfo *sockets, char *error)
+int32_t scap_fd_handle_socket(scap_t *handle, char *fname, scap_threadinfo *tinfo, scap_fdinfo *fdi, char* procdir, uint64_t net_ns, struct scap_ns_socket_list **sockets_by_ns, char *error)
 {
 	char link_name[1024];
 	ssize_t r;
 	scap_fdinfo *tfdi;
 	uint64_t ino;
+	struct scap_ns_socket_list* sockets = NULL;
+	int32_t uth_status = SCAP_SUCCESS;
 
-	if(sockets == NULL)
+	if(*sockets_by_ns == (void*)-1)
 	{
 		return SCAP_SUCCESS;
 	}
+	else 
+	{
+		HASH_FIND_INT64(*sockets_by_ns, &net_ns, sockets);
+		if(sockets == NULL)
+		{
+			sockets = malloc(sizeof(struct scap_ns_socket_list));
+			sockets->net_ns = net_ns;
+			sockets->sockets = NULL;
+
+			HASH_ADD_INT64(*sockets_by_ns, net_ns, sockets);
+			if(uth_status != SCAP_SUCCESS)
+			{
+				snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "socket list allocation error");
+				return SCAP_FAILURE;				
+			}
+
+			if(scap_fd_read_sockets(handle, procdir, sockets) == SCAP_FAILURE)
+			{
+				sockets->sockets = NULL;
+				return SCAP_FAILURE;
+			}
+		}
+	}
 
 	r = readlink(fname, link_name, 1024);
-	if (r <= 0)
+	if(r <= 0)
 	{
 		return SCAP_SUCCESS;
 	}
@@ -660,7 +702,7 @@ int32_t scap_fd_handle_socket(scap_t *handle, char *fname, scap_threadinfo *tinf
 	//
 	// Lookup ino in the list of sockets
 	//
-	HASH_FIND_INT64(sockets, &ino, tfdi);
+	HASH_FIND_INT64(sockets->sockets, &ino, tfdi);
 	if(tfdi != NULL)
 	{
 		memcpy(&(fdi->info), &(tfdi->info), sizeof(fdi->info));
@@ -674,7 +716,7 @@ int32_t scap_fd_handle_socket(scap_t *handle, char *fname, scap_threadinfo *tinf
 	}
 }
 
-int32_t scap_fd_read_unix_sockets_from_proc_fs(scap_t *handle, scap_fdinfo **sockets)
+int32_t scap_fd_read_unix_sockets_from_proc_fs(scap_t *handle, const char* filename, scap_fdinfo **sockets)
 {
 	FILE *f;
 	char line[1024];
@@ -682,7 +724,8 @@ int32_t scap_fd_read_unix_sockets_from_proc_fs(scap_t *handle, scap_fdinfo **soc
 	char *delimiters = " \t";
 	char *token;
 	int32_t uth_status = SCAP_SUCCESS;
-	f = fopen("/proc/net/unix", "r");
+
+	f = fopen(filename, "r");
 	if(NULL == f)
 	{
 		ASSERT(false);
@@ -785,15 +828,15 @@ int32_t scap_fd_read_unix_sockets_from_proc_fs(scap_t *handle, scap_fdinfo **soc
 		HASH_ADD_INT64((*sockets), ino, fdinfo);
 		if(uth_status != SCAP_SUCCESS)
 		{
-			// TODO: set some error message
-			break;
+			snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "unix socket allocatiallocation error");
+			return SCAP_FAILURE;
 		}
 	}
 	fclose(f);
 	return uth_status;
 }
 
-int32_t scap_fd_read_ipv4_sockets_from_proc_fs(scap_t *handle, char *dir, int l4proto, scap_fdinfo **sockets)
+int32_t scap_fd_read_ipv4_sockets_from_proc_fs(scap_t *handle, const char *dir, int l4proto, scap_fdinfo **sockets)
 {
 	FILE *f;
 	int32_t uth_status = SCAP_SUCCESS;
@@ -1185,27 +1228,79 @@ int32_t scap_fd_read_ipv6_sockets_from_proc_fs(scap_t *handle, char *dir, int l4
 	return uth_status;
 }
 
-int32_t scap_fd_read_sockets(scap_t *handle, scap_fdinfo **sockets)
+int32_t scap_fd_read_sockets(scap_t *handle, char* procdir, struct scap_ns_socket_list *sockets)
 {
-	if(SCAP_FAILURE == scap_fd_read_ipv4_sockets_from_proc_fs(handle, "/proc/net/tcp", SCAP_L4_TCP, sockets) ||
-	        SCAP_FAILURE == scap_fd_read_ipv4_sockets_from_proc_fs(handle, "/proc/net/udp", SCAP_L4_UDP, sockets) ||
-	        SCAP_FAILURE == scap_fd_read_ipv4_sockets_from_proc_fs(handle, "/proc/net/raw", SCAP_L4_RAW, sockets) ||
-	        SCAP_FAILURE == scap_fd_read_unix_sockets_from_proc_fs(handle, sockets))
+	char filename[SCAP_MAX_PATH_SIZE];
+	char netroot[SCAP_MAX_PATH_SIZE];
+
+	if(sockets->net_ns)
 	{
-		scap_fd_free_table(handle, sockets);
+		//
+		// Namespace support, look in /proc/PID/net/
+		//
+		snprintf(netroot, sizeof(netroot), "%snet/", procdir);
+	}
+	else
+	{
+		//
+		// No namespace support, look in the base /proc
+		//
+		snprintf(netroot, sizeof(netroot), "%s/proc/net/", scap_get_host_root());
+	}
+
+	snprintf(filename, sizeof(filename), "%stcp", netroot);
+	if(scap_fd_read_ipv4_sockets_from_proc_fs(handle, filename, SCAP_L4_TCP, &sockets->sockets) == SCAP_FAILURE)
+	{
+		scap_fd_free_table(handle, &sockets->sockets);
+		return SCAP_FAILURE;		
+	}
+
+	snprintf(filename, sizeof(filename), "%sudp", netroot);
+	if(scap_fd_read_ipv4_sockets_from_proc_fs(handle, filename, SCAP_L4_UDP, &sockets->sockets) == SCAP_FAILURE)
+	{
+		scap_fd_free_table(handle, &sockets->sockets);
+		return SCAP_FAILURE;		
+	}
+
+	snprintf(filename, sizeof(filename), "%sraw", netroot);
+	if(scap_fd_read_ipv4_sockets_from_proc_fs(handle, filename, SCAP_L4_RAW, &sockets->sockets) == SCAP_FAILURE)
+	{
+		scap_fd_free_table(handle, &sockets->sockets);
+		return SCAP_FAILURE;		
+	}
+
+	snprintf(filename, sizeof(filename), "%sunix", netroot);
+	if(scap_fd_read_unix_sockets_from_proc_fs(handle, filename, &sockets->sockets) == SCAP_FAILURE)
+	{
+		scap_fd_free_table(handle, &sockets->sockets);
 		return SCAP_FAILURE;
 	}
 
+	snprintf(filename, sizeof(filename), "%stcp6", netroot);
     /* We assume if there is /proc/net/tcp6 that ipv6 is avaiable */
-    if(0 == access("/proc/net/tcp6", R_OK)) {
-        if(SCAP_FAILURE == scap_fd_read_ipv6_sockets_from_proc_fs(handle, "/proc/net/tcp6", SCAP_L4_TCP, sockets) ||
-        	SCAP_FAILURE == scap_fd_read_ipv6_sockets_from_proc_fs(handle, "/proc/net/udp6", SCAP_L4_UDP, sockets) ||
-	        SCAP_FAILURE == scap_fd_read_ipv6_sockets_from_proc_fs(handle, "/proc/net/raw6", SCAP_L4_RAW, sockets))
-    	{
-	    	scap_fd_free_table(handle, sockets);
-		    return SCAP_FAILURE;
-    	}
+    if(access(filename, R_OK) == 0)
+    {
+		if(scap_fd_read_ipv6_sockets_from_proc_fs(handle, filename, SCAP_L4_TCP, &sockets->sockets) == SCAP_FAILURE)
+		{
+			scap_fd_free_table(handle, &sockets->sockets);
+			return SCAP_FAILURE;		
+		}
+
+		snprintf(filename, sizeof(filename), "%sudp6", netroot);
+		if(scap_fd_read_ipv6_sockets_from_proc_fs(handle, filename, SCAP_L4_TCP, &sockets->sockets) == SCAP_FAILURE)
+		{
+			scap_fd_free_table(handle, &sockets->sockets);
+			return SCAP_FAILURE;		
+		}
+
+		snprintf(filename, sizeof(filename), "%sraw6", netroot);
+		if(scap_fd_read_ipv6_sockets_from_proc_fs(handle, filename, SCAP_L4_TCP, &sockets->sockets) == SCAP_FAILURE)
+		{
+			scap_fd_free_table(handle, &sockets->sockets);
+			return SCAP_FAILURE;		
+		}
     }
+    
 	return SCAP_SUCCESS;
 }
 
@@ -1264,16 +1359,19 @@ char * decode_st_mode(struct stat* sb)
 //
 // Scan the directory containing the fd's of a proc /proc/x/fd
 //
-int32_t scap_fd_scan_fd_dir(scap_t *handle, char *procdir, scap_threadinfo *tinfo, scap_fdinfo *sockets, char *error)
+int32_t scap_fd_scan_fd_dir(scap_t *handle, char *procdir, scap_threadinfo *tinfo, struct scap_ns_socket_list **sockets_by_ns, char *error)
 {
 	DIR *dir_p;
 	struct dirent *dir_entry_p;
 	int32_t res = SCAP_SUCCESS;
 	char fd_dir_name[1024];
 	char f_name[1024];
+	char link_name[1024];
 	struct stat sb;
 	uint64_t fd;
 	scap_fdinfo *fdi = NULL;
+	uint64_t net_ns;
+	ssize_t r;
 
 	snprintf(fd_dir_name, 1024, "%sfd", procdir);
 	dir_p = opendir(fd_dir_name);
@@ -1283,6 +1381,24 @@ int32_t scap_fd_scan_fd_dir(scap_t *handle, char *procdir, scap_threadinfo *tinf
 		return SCAP_NOTFOUND;
 	}
 
+	//
+	// Get the network namespace of the process
+	//
+	snprintf(f_name, sizeof(f_name), "%sns/net", procdir);
+	r = readlink(f_name, link_name, sizeof(link_name));
+	if(r <= 0)
+	{
+		//
+		// No network namespace available. Assume global
+		//
+		net_ns = 0;
+	}
+	else
+	{
+		link_name[r] = '\0';
+		sscanf(link_name, "net:[%"PRIi64"]", &net_ns);
+	}
+
 	while((dir_entry_p = readdir(dir_p)) != NULL)
 	{
 		fdi = NULL;
@@ -1329,12 +1445,15 @@ int32_t scap_fd_scan_fd_dir(scap_t *handle, char *procdir, scap_threadinfo *tinf
 			{
 				break;
 			}
-			res = scap_fd_handle_socket(handle, f_name, tinfo, fdi, sockets, error);
-			if(fdi->type == SCAP_FD_UNKNOWN)
+			res = scap_fd_handle_socket(handle, f_name, tinfo, fdi, procdir, net_ns, sockets_by_ns, error);
+			if(handle->m_proc_callback == NULL)
 			{
 				// we can land here if we've got a netlink socket
-				scap_fd_free_fdinfo(&fdi);
-			}
+				if(fdi->type == SCAP_FD_UNKNOWN)
+				{
+					scap_fd_free_fdinfo(&fdi);
+				}
+			} 
 			break;
 		default:
 			res = scap_fd_allocate_fdinfo(handle, &fdi, fd, SCAP_FD_UNSUPPORTED);
@@ -1346,10 +1465,15 @@ int32_t scap_fd_scan_fd_dir(scap_t *handle, char *procdir, scap_threadinfo *tinf
 			res = scap_fd_handle_regular_file(handle, f_name, tinfo, fdi, error);
 			break;
 		}
-		if(NULL != fdi)
+
+		if(handle->m_proc_callback != NULL)
 		{
-			ASSERT(SCAP_FD_UNKNOWN != fdi->type);
+			if(fdi)
+			{
+				scap_fd_free_fdinfo(&fdi);
+			}
 		}
+
 		if(SCAP_SUCCESS != res)
 		{
 			break;
