@@ -160,7 +160,6 @@ static int32_t scap_write_proclist(scap_t *handle, gzFile f)
 	uint16_t commlen;
 	uint16_t exelen;
 	uint16_t argslen;
-	uint16_t envlen;
 	uint16_t cwdlen;
 
 	//
@@ -185,13 +184,16 @@ static int32_t scap_write_proclist(scap_t *handle, gzFile f)
 		    sizeof(uint64_t) +  // pfmajor
 		    sizeof(uint64_t) +  // pfminor
 		    2 + tinfo->env_len +
+		    sizeof(int64_t) +  // vtid
+		    sizeof(int64_t) +  // vpid
+		    2 + tinfo->cgroups_len +
 		    sizeof(uint32_t));
 	}
 
 	//
 	// Create the block
 	//
-	bh.block_type = PL_BLOCK_TYPE_V3;
+	bh.block_type = PL_BLOCK_TYPE_V4;
 	bh.block_total_length = scap_normalize_block_len(sizeof(block_header) + totlen + 4);
 
 	if(gzwrite(f, &bh, sizeof(bh)) != sizeof(bh))
@@ -208,7 +210,6 @@ static int32_t scap_write_proclist(scap_t *handle, gzFile f)
 		commlen = (uint16_t)strnlen(tinfo->comm, SCAP_MAX_PATH_SIZE);
 		exelen = (uint16_t)strnlen(tinfo->exe, SCAP_MAX_PATH_SIZE);
 		argslen = tinfo->args_len;
-		envlen = tinfo->env_len;
 		cwdlen = (uint16_t)strnlen(tinfo->cwd, SCAP_MAX_PATH_SIZE);
 
 		if(gzwrite(f, &(tinfo->tid), sizeof(uint64_t)) != sizeof(uint64_t) ||
@@ -231,8 +232,12 @@ static int32_t scap_write_proclist(scap_t *handle, gzFile f)
 		        gzwrite(f, &(tinfo->vmswap_kb), sizeof(uint32_t)) != sizeof(uint32_t) ||
 		        gzwrite(f, &(tinfo->pfmajor), sizeof(uint64_t)) != sizeof(uint64_t) ||
 		        gzwrite(f, &(tinfo->pfminor), sizeof(uint64_t)) != sizeof(uint64_t) ||
-		        gzwrite(f, &envlen, sizeof(uint16_t)) != sizeof(uint16_t) ||
-		        gzwrite(f, tinfo->env, envlen) != envlen)
+		        gzwrite(f, &(tinfo->env_len), sizeof(uint16_t)) != sizeof(uint16_t) ||
+		        gzwrite(f, tinfo->env, tinfo->env_len) != tinfo->env_len ||
+		        gzwrite(f, &(tinfo->vtid), sizeof(int64_t)) != sizeof(int64_t) ||
+		        gzwrite(f, &(tinfo->vpid), sizeof(int64_t)) != sizeof(int64_t) ||
+		        gzwrite(f, &(tinfo->cgroups_len), sizeof(uint16_t)) != sizeof(uint16_t) ||
+		        gzwrite(f, tinfo->cgroups, tinfo->cgroups_len) != tinfo->cgroups_len)
 		{
 			snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "error writing to file (2)");
 			return SCAP_FAILURE;
@@ -546,11 +551,19 @@ static scap_dumper_t *scap_setup_dump(scap_t *handle, gzFile f, const char *fnam
 #if defined(HAS_CAPTURE)
 	if(handle->m_file == NULL)
 	{
+		proc_entry_callback tcb = handle->m_proc_callback;
+		handle->m_proc_callback = NULL;
+
 		scap_proc_free_table(handle);
-		if(scap_proc_scan_proc_dir(handle, "/proc", -1, -1, NULL, handle->m_lasterr, true) != SCAP_SUCCESS)
+		char filename[SCAP_MAX_PATH_SIZE];
+		snprintf(filename, sizeof(filename), "%s/proc", scap_get_host_root());
+		if(scap_proc_scan_proc_dir(handle, filename, -1, -1, NULL, handle->m_lasterr, true) != SCAP_SUCCESS)
 		{
+			handle->m_proc_callback = tcb;
 			return NULL;
 		}
+
+		handle->m_proc_callback = tcb;
 	}
 #endif
 
@@ -595,6 +608,14 @@ static scap_dumper_t *scap_setup_dump(scap_t *handle, gzFile f, const char *fnam
 		return NULL;
 	}
 
+	//
+	// If the user doesn't need the thread table, free it
+	//
+	if(handle->m_proc_callback != NULL)
+	{
+		scap_proc_free_table(handle);
+	}
+
 	//
 	// Done, return the file
 	//
@@ -794,6 +815,9 @@ static int32_t scap_read_proclist(scap_t *handle, gzFile f, uint32_t block_lengt
 	tinfo.pfmajor = 0;
 	tinfo.pfminor = 0;
 	tinfo.env_len = 0;
+	tinfo.vtid = -1;
+	tinfo.vpid = -1;
+	tinfo.cgroups_len = 0;
 
 	while(((int32_t)block_length - (int32_t)totreadsize) >= 4)
 	{
@@ -827,7 +851,7 @@ static int32_t scap_read_proclist(scap_t *handle, gzFile f, uint32_t block_lengt
 		readsize = gzread(f, &(stlen), sizeof(uint16_t));
 		CHECK_READ_SIZE(readsize, sizeof(uint16_t));
 
-		if(stlen >= SCAP_MAX_PATH_SIZE)
+		if(stlen > SCAP_MAX_PATH_SIZE)
 		{
 			snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "invalid commlen %d", stlen);
 			return SCAP_FAILURE;
@@ -849,7 +873,7 @@ static int32_t scap_read_proclist(scap_t *handle, gzFile f, uint32_t block_lengt
 		readsize = gzread(f, &(stlen), sizeof(uint16_t));
 		CHECK_READ_SIZE(readsize, sizeof(uint16_t));
 
-		if(stlen >= SCAP_MAX_PATH_SIZE)
+		if(stlen > SCAP_MAX_PATH_SIZE)
 		{
 			snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "invalid exelen %d", stlen);
 			return SCAP_FAILURE;
@@ -871,7 +895,7 @@ static int32_t scap_read_proclist(scap_t *handle, gzFile f, uint32_t block_lengt
 		readsize = gzread(f, &(stlen), sizeof(uint16_t));
 		CHECK_READ_SIZE(readsize, sizeof(uint16_t));
 
-		if(stlen >= SCAP_MAX_PATH_SIZE)
+		if(stlen > SCAP_MAX_ARGS_SIZE)
 		{
 			snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "invalid argslen %d", stlen);
 			return SCAP_FAILURE;
@@ -894,7 +918,7 @@ static int32_t scap_read_proclist(scap_t *handle, gzFile f, uint32_t block_lengt
 		readsize = gzread(f, &(stlen), sizeof(uint16_t));
 		CHECK_READ_SIZE(readsize, sizeof(uint16_t));
 
-		if(stlen >= SCAP_MAX_PATH_SIZE)
+		if(stlen > SCAP_MAX_PATH_SIZE)
 		{
 			snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "invalid cwdlen %d", stlen);
 			return SCAP_FAILURE;
@@ -951,6 +975,7 @@ static int32_t scap_read_proclist(scap_t *handle, gzFile f, uint32_t block_lengt
 		case PL_BLOCK_TYPE_V2_INT:
 		case PL_BLOCK_TYPE_V3:
 		case PL_BLOCK_TYPE_V3_INT:
+		case PL_BLOCK_TYPE_V4:
 			//
 			// vmsize_kb
 			//
@@ -992,15 +1017,16 @@ static int32_t scap_read_proclist(scap_t *handle, gzFile f, uint32_t block_lengt
 			totreadsize += readsize;
 
 			if(block_type == PL_BLOCK_TYPE_V3 ||
-				block_type == PL_BLOCK_TYPE_V3_INT)
+				block_type == PL_BLOCK_TYPE_V3_INT ||
+				block_type == PL_BLOCK_TYPE_V4)
 			{
 				//
-				// args
+				// env
 				//
 				readsize = gzread(f, &(stlen), sizeof(uint16_t));
 				CHECK_READ_SIZE(readsize, sizeof(uint16_t));
 
-				if(stlen >= SCAP_MAX_ENV_SIZE)
+				if(stlen > SCAP_MAX_ENV_SIZE)
 				{
 					snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "invalid envlen %d", stlen);
 					return SCAP_FAILURE;
@@ -1017,6 +1043,45 @@ static int32_t scap_read_proclist(scap_t *handle, gzFile f, uint32_t block_lengt
 
 				totreadsize += readsize;
 			}
+
+			if(block_type == PL_BLOCK_TYPE_V4)
+			{
+				//
+				// vtid
+				//
+				readsize = gzread(f, &(tinfo.vtid), sizeof(int64_t));
+				CHECK_READ_SIZE(readsize, sizeof(uint64_t));
+
+				totreadsize += readsize;
+
+				//
+				// vpid
+				//
+				readsize = gzread(f, &(tinfo.vpid), sizeof(int64_t));
+				CHECK_READ_SIZE(readsize, sizeof(uint64_t));
+
+				totreadsize += readsize;
+
+				//
+				// cgroups
+				//
+				readsize = gzread(f, &(stlen), sizeof(uint16_t));
+				CHECK_READ_SIZE(readsize, sizeof(uint16_t));
+
+				if(stlen > SCAP_MAX_CGROUPS_SIZE)
+				{
+					snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "invalid cgroupslen %d", stlen);
+					return SCAP_FAILURE;
+				}
+				tinfo.cgroups_len = stlen;
+
+				totreadsize += readsize;
+
+				readsize = gzread(f, tinfo.cgroups, stlen);
+				CHECK_READ_SIZE(readsize, stlen);
+
+				totreadsize += readsize;
+			}
 			break;
 		default:
 			snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "corrupted process block type (fd1)");
@@ -1025,27 +1090,34 @@ static int32_t scap_read_proclist(scap_t *handle, gzFile f, uint32_t block_lengt
 		}
 
 		//
-		// All parsed. Allocate the new entry and copy the temp one into into it.
+		// All parsed. Add the entry to the table, or fire the notification callback
 		//
-		ntinfo = (scap_threadinfo *)malloc(sizeof(scap_threadinfo));
-		if(ntinfo == NULL)
+		if(handle->m_proc_callback == NULL)
 		{
-			snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "process table allocation error (fd1)");
-			return SCAP_FAILURE;
+			//
+			// All parsed. Allocate the new entry and copy the temp one into into it.
+			//
+			ntinfo = (scap_threadinfo *)malloc(sizeof(scap_threadinfo));
+			if(ntinfo == NULL)
+			{
+				snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "process table allocation error (fd1)");
+				return SCAP_FAILURE;
+			}
+
+			// Structure copy
+			*ntinfo = tinfo;
+
+			HASH_ADD_INT64(handle->m_proclist, tid, ntinfo);
+			if(uth_status != SCAP_SUCCESS)
+			{
+				free(ntinfo);
+				snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "process table allocation error (fd2)");
+				return SCAP_FAILURE;
+			}
 		}
-
-		// Structure copy
-		*ntinfo = tinfo;
-
-		//
-		// All parsed. Add the entry to the table
-		//
-		HASH_ADD_INT64(handle->m_proclist, tid, ntinfo);
-		if(uth_status != SCAP_SUCCESS)
+		else
 		{
-			free(ntinfo);
-			snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "process table allocation error (fd2)");
-			return SCAP_FAILURE;
+			handle->m_proc_callback(handle->m_proc_callback_context, tinfo.tid, &tinfo, NULL, handle);
 		}
 	}
 
@@ -1381,6 +1453,7 @@ static int32_t scap_read_iflist(scap_t *handle, gzFile f, uint32_t block_length)
 
 scap_read_iflist_error:
 	scap_free_iflist(handle->m_addrlist);
+	handle->m_addrlist = NULL;
 
 	if(readbuf)
 	{
@@ -1622,15 +1695,22 @@ static int32_t scap_read_fdlist(scap_t *handle, gzFile f, uint32_t block_length)
 	CHECK_READ_SIZE(readsize, sizeof(tid));
 	totreadsize += readsize;
 
-	//
-	// Identify the process descriptor
-	//
-	HASH_FIND_INT64(handle->m_proclist, &tid, tinfo);
-	if(tinfo == NULL)
+	if(handle->m_proc_callback == NULL)
 	{
-		snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "corrupted trace file. FD block references TID %"PRIu64", which doesn't exist.",
-		         tid);
-		return SCAP_FAILURE;
+		//
+		// Identify the process descriptor
+		//
+		HASH_FIND_INT64(handle->m_proclist, &tid, tinfo);
+		if(tinfo == NULL)
+		{
+			snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "corrupted trace file. FD block references TID %"PRIu64", which doesn't exist.",
+					 tid);
+			return SCAP_FAILURE;
+		}
+	}
+	else
+	{
+		tinfo = NULL;
 	}
 
 	while(((int32_t)block_length - (int32_t)totreadsize) >= 4)
@@ -1642,27 +1722,38 @@ static int32_t scap_read_fdlist(scap_t *handle, gzFile f, uint32_t block_length)
 		totreadsize += readsize;
 
 		//
-		// Parsed successfully. Allocate the new entry and copy the temp one into into it.
+		// Add the entry to the table, or fire the notification callback
 		//
-		nfdi = (scap_fdinfo *)malloc(sizeof(scap_fdinfo));
-		if(nfdi == NULL)
+		if(handle->m_proc_callback == NULL)
 		{
-			snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "process table allocation error (fd1)");
-			return SCAP_FAILURE;
+			//
+			// Parsed successfully. Allocate the new entry and copy the temp one into into it.
+			//
+			nfdi = (scap_fdinfo *)malloc(sizeof(scap_fdinfo));
+			if(nfdi == NULL)
+			{
+				snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "process table allocation error (fd1)");
+				return SCAP_FAILURE;
+			}
+
+			// Structure copy
+			*nfdi = fdi;
+
+			ASSERT(tinfo != NULL);
+
+			HASH_ADD_INT64(tinfo->fdlist, fd, nfdi);
+			if(uth_status != SCAP_SUCCESS)
+			{
+				free(nfdi);
+				snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "process table allocation error (fd2)");
+				return SCAP_FAILURE;
+			}
 		}
-
-		// Structure copy
-		*nfdi = fdi;
-
-		//
-		// Add the entry to the table
-		//
-		HASH_ADD_INT64(tinfo->fdlist, fd, nfdi);
-		if(uth_status != SCAP_SUCCESS)
+		else
 		{
-			free(nfdi);
-			snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "process table allocation error (fd2)");
-			return SCAP_FAILURE;
+			ASSERT(tinfo == NULL);
+
+			handle->m_proc_callback(handle->m_proc_callback_context, tid, NULL, &fdi, handle);
 		}
 	}
 
@@ -1757,6 +1848,7 @@ int32_t scap_read_init(scap_t *handle, gzFile f)
 		case PL_BLOCK_TYPE_V1:
 		case PL_BLOCK_TYPE_V2:
 		case PL_BLOCK_TYPE_V3:
+		case PL_BLOCK_TYPE_V4:
 		case PL_BLOCK_TYPE_V1_INT:
 		case PL_BLOCK_TYPE_V2_INT:
 		case PL_BLOCK_TYPE_V3_INT:
