@@ -24,6 +24,7 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 #include <limits.h>
 #include <stdlib.h>
 #include <unistd.h>
+#include <sys/ioctl.h>
 #endif
 #include <third-party/tinydir.h>
 #include <json/json.h>
@@ -89,6 +90,8 @@ uint32_t lua_cbacks::rawval_to_lua_stack(lua_State *ls, uint8_t* rawval, const f
 			return 1;
 		case PT_FLAGS32:
 		case PT_UINT32:
+		case PT_UID:
+		case PT_GID:
 			lua_pushnumber(ls, *(uint32_t*)rawval);
 			return 1;
 		case PT_UINT64:
@@ -534,6 +537,31 @@ int lua_cbacks::is_tty(lua_State *ls)
 	return 1;
 }
 
+int lua_cbacks::get_terminal_info(lua_State *ls) 
+{
+	int32_t width = -1;
+	int32_t height = -1;
+#ifndef _WIN32
+	struct winsize w;
+
+	if(ioctl(STDOUT_FILENO, TIOCGWINSZ, &w) != -1)
+	{
+		width = w.ws_col;
+		height = w.ws_row;
+	}
+#endif
+
+	lua_newtable(ls);
+	lua_pushstring(ls, "width");
+	lua_pushnumber(ls, width);
+	lua_settable(ls, -3);
+	lua_pushstring(ls, "height");
+	lua_pushnumber(ls, height);
+	lua_settable(ls, -3);
+
+	return 1;
+}
+
 int lua_cbacks::get_filter(lua_State *ls) 
 {
 	lua_getglobal(ls, "sichisel");
@@ -694,9 +722,6 @@ int lua_cbacks::get_thread_table(lua_State *ls)
 		lua_pushliteral(ls, "ptid");
 		lua_pushnumber(ls, (uint32_t)it->second.m_ptid);
 		lua_settable(ls, -3);
-		lua_pushliteral(ls, "progid");
-		lua_pushnumber(ls, (uint32_t)it->second.m_progid);
-		lua_settable(ls, -3);
 		lua_pushliteral(ls, "comm");
 		lua_pushstring(ls, it->second.m_comm.c_str());
 		lua_settable(ls, -3);
@@ -755,7 +780,6 @@ int lua_cbacks::get_thread_table(lua_State *ls)
 			uit = userlist->find(it->second.m_uid);
 			if(uit == userlist->end())
 			{
-				ASSERT(false);
 				username = "<NA>";
 			}
 			else
@@ -957,6 +981,94 @@ int lua_cbacks::get_thread_table(lua_State *ls)
 	return 1;
 }
 
+int lua_cbacks::get_container_table(lua_State *ls) 
+{
+	unordered_map<int64_t, sinsp_fdinfo_t>::iterator fdit;
+	uint32_t j;
+	sinsp_evt tevt;
+
+	//
+	// Get the chisel state
+	//
+	lua_getglobal(ls, "sichisel");
+
+	sinsp_chisel* ch = (sinsp_chisel*)lua_touserdata(ls, -1);
+	lua_pop(ls, 1);
+
+	ASSERT(ch);
+	ASSERT(ch->m_lua_cinfo);
+	ASSERT(ch->m_inspector);
+
+	//
+	// Retrieve the container list
+	//
+	const unordered_map<string, sinsp_container_info>* ctable  = ch->m_inspector->m_container_manager.get_containers();
+
+	ASSERT(ctable != NULL);
+
+	lua_newtable(ls);
+
+	//
+	// Go through the list
+	//
+	j = 0;
+	for(auto it = ctable->begin(); it != ctable->end(); ++it)
+	{
+		lua_newtable(ls);
+		lua_pushliteral(ls, "id");
+		lua_pushstring(ls, it->second.m_id.c_str());
+		lua_settable(ls, -3);
+		lua_pushliteral(ls, "name");
+		lua_pushstring(ls, it->second.m_name.c_str());
+		lua_settable(ls, -3);
+		lua_pushliteral(ls, "image");
+		lua_pushstring(ls, it->second.m_image.c_str());
+		lua_settable(ls, -3);
+
+		lua_pushliteral(ls, "type");
+		if(it->second.m_type == CT_DOCKER)
+		{
+			lua_pushstring(ls, "docker");
+		}
+		else if(it->second.m_type == CT_LXC)
+		{
+			lua_pushstring(ls, "lxc");
+		}
+		else if(it->second.m_type == CT_LIBVIRT_LXC)
+		{
+			lua_pushstring(ls, "libvirt_lxc");
+		}
+		else
+		{
+			ASSERT(false);
+			lua_pushstring(ls, "unknown");
+		}
+		lua_settable(ls, -3);
+
+		//
+		// Set the key for this entry
+		//
+		lua_rawseti(ls,-2, (uint32_t)++j);
+	}
+
+	return 1;
+}
+
+int lua_cbacks::is_print_container_data(lua_State *ls) 
+{
+        lua_getglobal(ls, "sichisel");
+
+        sinsp_chisel* ch = (sinsp_chisel*)lua_touserdata(ls, -1);
+        lua_pop(ls, 1);
+
+        ASSERT(ch);
+        ASSERT(ch->m_lua_cinfo);
+
+        lua_pushboolean(ls, ch->m_inspector->is_print_container_data());
+        return 1;
+}
+
+
 int lua_cbacks::get_output_format(lua_State *ls) 
 {
 	lua_getglobal(ls, "sichisel");
