@@ -41,9 +41,12 @@ public:
 	static int end_capture(lua_State *ls);
 	static int is_live(lua_State *ls);
 	static int is_tty(lua_State *ls);
+	static int get_terminal_info(lua_State *ls);
 	static int get_filter(lua_State *ls);
 	static int get_machine_info(lua_State *ls);
 	static int get_thread_table(lua_State *ls);
+	static int get_container_table(lua_State *ls);
+	static int is_print_container_data(lua_State *ls);
 	static int get_output_format(lua_State *ls);
 	static int get_evtsource_name(lua_State *ls);
 	static int set_event_formatter(lua_State *ls);
