@@ -74,7 +74,11 @@ public:
 	char* m_val;	///< Pointer to the event parameter data.
 	uint16_t m_len; ///< Lenght os the parameter pointed by m_val.
 private:
-	void init(char* valptr, uint16_t len);
+	inline void init(char* valptr, uint16_t len)
+	{
+		m_val = valptr;
+		m_len = len;		
+	}
 
 	friend class sinsp_evt;
 };
@@ -94,12 +98,17 @@ public:
 	*/
 	enum param_fmt
 	{
-		PF_NORMAL =   (1 << 0),	///< Normal screen output
-		PF_JSON =     (1 << 1),	///< Json formatting
-		PF_SIMPLE =   (1 << 2),	///< Reduced output, e.g. not type character for FDs
-		PF_HEX =      (1 << 3),	///< Hexadecimal output
-		PF_HEXASCII = (1 << 4),	///< Hexadecimal + ASCII output
-		PF_EOLS =     (1 << 5),	///< Normal + end of lines
+		PF_NORMAL =         (1 << 0),	///< Normal screen output
+		PF_JSON =           (1 << 1),	///< Json formatting with data in normal screen format
+		PF_SIMPLE =         (1 << 2),	///< Reduced output, e.g. not type character for FDs
+		PF_HEX =            (1 << 3),	///< Hexadecimal output
+		PF_HEXASCII =       (1 << 4),	///< Hexadecimal + ASCII output
+		PF_EOLS =           (1 << 5),	///< Normal + end of lines
+		PF_BASE64 =         (1 << 6),	///< Base64 output
+		PF_JSONEOLS =       (1 << 7),	///< Json formatting with data in hexadecimal format
+		PF_JSONHEX =        (1 << 8),	///< Json formatting with data in hexadecimal format
+		PF_JSONHEXASCII =   (1 << 9),	///< Json formatting with data in hexadecimal + ASCII format
+		PF_JSONBASE64 =     (1 << 10),	///< Json formatting with data in base64 format
 	};
 
 	/*!
@@ -136,24 +145,36 @@ public:
 	/*!
 	  \brief Get the incremental number of this event.
 	*/
-	uint64_t get_num();
+	inline uint64_t get_num()
+	{
+		return m_evtnum;
+	}
 
 	/*!
 	  \brief Get the number of the CPU where this event was captured.
 	*/
-	int16_t get_cpuid();
+	inline int16_t get_cpuid()
+	{
+		return m_cpuid;
+	}
 
 	/*!
 	  \brief Get the event type. 
 	  
 	  \note For a list of event types, refer to \ref etypes.
 	*/
-	uint16_t get_type();
+	inline uint16_t get_type()
+	{
+		return m_pevt->type;
+	}
 
 	/*!
 	  \brief Get the event's flags.
 	*/
-	ppm_event_flags get_flags();
+	inline ppm_event_flags get_flags()
+	{
+		return m_info->flags;
+	}
 
 	/*!
 	  \brief Return the event direction: in or out.
@@ -172,6 +193,14 @@ public:
 	*/
 	const char* get_name();
 
+	/*!
+	  \brief Return the event category.
+	*/
+	inline ppm_event_category get_category()
+	{
+		return m_info->category;
+	}
+
 	/*!
 	  \brief Get the ID of the thread that generated the event.
 	*/
@@ -272,9 +301,43 @@ private:
 
 	const char* get_param_value_str(const char* name, OUT const char** resolved_str, param_fmt fmt = PF_NORMAL);
 
-	void init();
-	void init(uint8_t* evdata, uint16_t cpuid);
-	void load_params();
+	inline void init()
+	{
+		m_params_loaded = false;
+		m_info = &(m_event_info_table[m_pevt->type]);
+		m_tinfo = NULL;
+		m_fdinfo = NULL;
+		m_iosize = 0;		
+	}
+	inline void init(uint8_t* evdata, uint16_t cpuid)
+	{
+		m_params_loaded = false;
+		m_pevt = (scap_evt *)evdata;
+		m_info = &(m_event_info_table[m_pevt->type]);
+		m_tinfo = NULL;
+		m_fdinfo = NULL;
+		m_iosize = 0;
+		m_cpuid = cpuid;
+		m_evtnum = 0;		
+	}
+	inline void load_params()
+	{
+		uint32_t j;
+		uint32_t nparams;
+		sinsp_evt_param par;
+
+		nparams = m_info->nparams;
+		uint16_t *lens = (uint16_t *)((char *)m_pevt + sizeof(struct ppm_evt_hdr));
+		char *valptr = (char *)lens + nparams * sizeof(uint16_t);
+		m_params.clear();
+
+		for(j = 0; j < nparams; j++)
+		{
+			par.init(valptr, lens[j]);
+			m_params.push_back(par);
+			valptr += lens[j];
+		}
+	}
 	string get_param_value_str(uint32_t id, bool resolved);
 	string get_param_value_str(const char* name, bool resolved = true);
 	char* render_fd(int64_t fd, const char** resolved_str, sinsp_evt::param_fmt fmt);
@@ -302,6 +365,7 @@ VISIBILITY_PRIVATE
 #ifdef HAS_FILTERING
 	bool m_filtered_out;
 #endif
+	const struct ppm_event_info* m_event_info_table;
 
 	friend class sinsp;
 	friend class sinsp_parser;
@@ -313,6 +377,8 @@ VISIBILITY_PRIVATE
 	friend class sinsp_analyzer_fd_listener;
 	friend class sinsp_analyzer_parsers;
 	friend class lua_cbacks;
+	friend class sinsp_proto_detector;
+	friend class sinsp_container_manager;
 };
 
 /*@}*/
