@@ -59,7 +59,7 @@ sinsp_filter_check_fd::sinsp_filter_check_fd()
 
 	m_info.m_name = "fd";
 	m_info.m_fields = sinsp_filter_check_fd_fields;
-	m_info.m_nfiedls = sizeof(sinsp_filter_check_fd_fields) / sizeof(sinsp_filter_check_fd_fields[0]);
+	m_info.m_nfields = sizeof(sinsp_filter_check_fd_fields) / sizeof(sinsp_filter_check_fd_fields[0]);
 	m_info.m_flags = filter_check_info::FL_WORKS_ON_THREAD_TABLE;
 }
 
@@ -197,7 +197,7 @@ uint8_t* sinsp_filter_check_fd::extract_from_null_fd(sinsp_evt *evt, OUT uint32_
 			{
 				if(pos < m_tstr.size() - 1)
 				{
-					m_tstr.resize(pos + 1);
+					m_tstr.resize(pos);
 				}
 			}
 			else
@@ -362,7 +362,7 @@ uint8_t* sinsp_filter_check_fd::extract(sinsp_evt *evt, OUT uint32_t* len)
 				{
 					if(pos < m_tstr.size() - 1)
 					{
-						m_tstr.resize(pos + 1);
+						m_tstr.resize(pos);
 					}
 				}
 				else
@@ -813,7 +813,7 @@ bool sinsp_filter_check_fd::compare(sinsp_evt *evt)
 const filtercheck_field_info sinsp_filter_check_thread_fields[] =
 {
 	{PT_INT64, EPF_NONE, PF_DEC, "proc.pid", "the id of the process generating the event."},
-	{PT_CHARBUF, EPF_NONE, PF_NA, "proc.exe", "the full name (including the path) of the executable generating the event."},
+	{PT_CHARBUF, EPF_NONE, PF_NA, "proc.exe", "the first command line argument (usually the executable name or a custom one)."},
 	{PT_CHARBUF, EPF_NONE, PF_NA, "proc.name", "the name (excluding the path) of the executable generating the event."},
 	{PT_CHARBUF, EPF_NONE, PF_NA, "proc.args", "the arguments passed on the command line when starting the process generating the event."},
 	{PT_CHARBUF, EPF_NONE, PF_NA, "proc.env", "the environment variables of the process generating the event."},
@@ -838,6 +838,10 @@ const filtercheck_field_info sinsp_filter_check_thread_fields[] =
 	{PT_BOOL, EPF_NONE, PF_NA, "thread.ismain", "'true' if the thread generating the event is the main one in the process."},
 	{PT_RELTIME, EPF_NONE, PF_DEC, "thread.exectime", "CPU time spent by the last scheduled thread, in nanoseconds. Exported by switch events only."},
 	{PT_RELTIME, EPF_NONE, PF_DEC, "thread.totexectime", "Total CPU time, in nanoseconds since the beginning of the capture, for the current thread. Exported by switch events only."},
+	{PT_CHARBUF, EPF_NONE, PF_NA, "thread.cgroups", "all the cgroups the thread belongs to, aggregated into a single string."},
+	{PT_CHARBUF, EPF_NONE, PF_NA, "thread.cgroup", "the cgroup the thread belongs to, for a specific subsystem. E.g. thread.cgroup.cpuacct."},
+	{PT_INT64, EPF_NONE, PF_DEC, "thread.vtid", "the id of the thread generating the event as seen from its current PID namespace."},
+	{PT_INT64, EPF_NONE, PF_DEC, "proc.vpid", "the id of the process generating the event as seen from its current PID namespace."},
 //	{PT_UINT64, EPF_NONE, PF_DEC, "iobytes", "I/O bytes (either read or write) generated by I/O calls like read, write, send receive..."},
 //	{PT_UINT64, EPF_NONE, PF_DEC, "totiobytes", "aggregated number of I/O bytes (either read or write) since the beginning of the capture."},
 //	{PT_RELTIME, EPF_NONE, PF_DEC, "latency", "number of nanoseconds spent in the last system call."},
@@ -848,7 +852,7 @@ sinsp_filter_check_thread::sinsp_filter_check_thread()
 {
 	m_info.m_name = "process";
 	m_info.m_fields = sinsp_filter_check_thread_fields;
-	m_info.m_nfiedls = sizeof(sinsp_filter_check_thread_fields) / sizeof(sinsp_filter_check_thread_fields[0]);
+	m_info.m_nfields = sizeof(sinsp_filter_check_thread_fields) / sizeof(sinsp_filter_check_thread_fields[0]);
 	m_info.m_flags = filter_check_info::FL_WORKS_ON_THREAD_TABLE;
 
 	m_u64val = 0;
@@ -866,16 +870,41 @@ int32_t sinsp_filter_check_thread::extract_arg(string fldname, string val, OUT c
 	//
 	// 'arg' and 'resarg' are handled in a custom way
 	//
-	if(val[fldname.size()] == '[')
+	if(m_field_id == TYPE_APID || m_field_id == TYPE_ANAME)
 	{
-		parsed_len = (uint32_t)val.find(']');
-		string numstr = val.substr(fldname.size() + 1, parsed_len - fldname.size() - 1);
-		m_argid = sinsp_numparser::parsed32(numstr);
-		parsed_len++;
+		if(val[fldname.size()] == '[')
+		{
+			parsed_len = (uint32_t)val.find(']');
+			string numstr = val.substr(fldname.size() + 1, parsed_len - fldname.size() - 1);
+			m_argid = sinsp_numparser::parsed32(numstr);
+			parsed_len++;
+		}
+		else
+		{
+			throw sinsp_exception("filter syntax error: " + val);
+		}
 	}
-	else
+	else if(m_field_id == TYPE_CGROUP)
 	{
-		throw sinsp_exception("filter syntax error: " + val);
+		if(val[fldname.size()] == '.')
+		{
+			size_t endpos;
+			for(endpos = fldname.size() + 1; endpos < val.length(); ++endpos)
+			{
+				if(!isalpha(val[endpos]) 
+					&& val[endpos] != '_')
+				{
+					break;
+				}
+			}
+
+			parsed_len = endpos;
+			m_argname = val.substr(fldname.size() + 1, endpos - fldname.size() - 1);
+		}
+		else
+		{
+			throw sinsp_exception("filter syntax error: " + val);
+		}		
 	}
 
 	return parsed_len; 
@@ -944,6 +973,14 @@ int32_t sinsp_filter_check_thread::parse_field_name(const char* str)
 		m_th_state_id = m_inspector->reserve_thread_memory(sizeof(uint64_t));
 		return sinsp_filter_check::parse_field_name(str);
 	}
+	else if(string(val, 0, sizeof("thread.cgroup") - 1) == "thread.cgroup" &&
+			string(val, 0, sizeof("thread.cgroups") - 1) != "thread.cgroups")
+	{
+		m_field_id = TYPE_CGROUP;
+		m_field = &m_info.m_fields[m_field_id];
+
+		return extract_arg("thread.cgroup", val, NULL);
+	}
 	else
 	{
 		return sinsp_filter_check::parse_field_name(str);
@@ -1066,6 +1103,8 @@ uint8_t* sinsp_filter_check_thread::extract(sinsp_evt *evt, OUT uint32_t* len)
 	case TYPE_CWD:
 		m_tstr = tinfo->get_cwd();
 		return (uint8_t*)m_tstr.c_str();
+	case TYPE_NCHILDS:
+		return (uint8_t*)&tinfo->m_nchilds;
 	case TYPE_ISMAINTHREAD:
 		m_tbool = (uint32_t)tinfo->is_main_thread();
 		return (uint8_t*)&m_tbool;
@@ -1322,6 +1361,67 @@ uint8_t* sinsp_filter_check_thread::extract(sinsp_evt *evt, OUT uint32_t* len)
 	case TYPE_PFMINOR:
 		m_u64val = tinfo->m_pfminor;
 		return (uint8_t*)&m_u64val;
+	case TYPE_CGROUPS:
+		{
+			m_tstr.clear();
+
+			uint32_t j;
+			uint32_t nargs = tinfo->m_cgroups.size();
+
+			if(nargs == 0)
+			{
+				return NULL;
+			}
+			
+			for(j = 0; j < nargs; j++)
+			{
+				m_tstr += tinfo->m_cgroups[j].first;
+				m_tstr += "=";
+				m_tstr += tinfo->m_cgroups[j].second;				
+				if(j < nargs - 1)
+				{
+					m_tstr += ' ';
+				}
+			}
+
+			return (uint8_t*)m_tstr.c_str();
+		}
+	case TYPE_CGROUP:
+		{
+			uint32_t nargs = tinfo->m_cgroups.size();
+
+			if(nargs == 0)
+			{
+				return NULL;
+			}
+			
+			for(uint32_t j = 0; j < nargs; j++)
+			{
+				if(tinfo->m_cgroups[j].first == m_argname)
+				{
+					m_tstr = tinfo->m_cgroups[j].second;
+					return (uint8_t*)m_tstr.c_str();					
+				}
+			}
+
+			return NULL;
+		}
+	case TYPE_VTID:
+		if(tinfo->m_vtid == -1)
+		{
+			return NULL;
+		}
+
+		m_u64val = tinfo->m_vtid;
+		return (uint8_t*)&m_u64val;
+	case TYPE_VPID:
+		if(tinfo->m_vpid == -1)
+		{
+			return NULL;
+		}
+
+		m_u64val = tinfo->m_vpid;
+		return (uint8_t*)&m_u64val;
 	default:
 		ASSERT(false);
 		return NULL;
@@ -1477,6 +1577,7 @@ const filtercheck_field_info sinsp_filter_check_event_fields[] =
 	{PT_DYN, EPF_REQUIRES_ARGUMENT, PF_NA, "evt.rawarg", "one of the event arguments specified by name. E.g. 'arg.fd'."},
 	{PT_CHARBUF, EPF_NONE, PF_NA, "evt.info", "for most events, this field returns the same value as evt.args. However, for some events (like writes to /dev/log) it provides higher level information coming from decoding the arguments."},
 	{PT_BYTEBUF, EPF_NONE, PF_NA, "evt.buffer", "the binary data buffer for events that have one, like read(), recvfrom(), etc. Use this field in filters with 'contains' to search into I/O data buffers."},
+	{PT_UINT64, EPF_NONE, PF_DEC, "evt.buflen", "the lenght of the binary data buffer for events that have one, like read(), recvfrom(), etc."},
 	{PT_CHARBUF, EPF_NONE, PF_DEC, "evt.res", "event return value, as an error code string (e.g. 'ENOENT')."},
 	{PT_INT64, EPF_NONE, PF_DEC, "evt.rawres", "event return value, as a number (e.g. -2). Useful for range comparisons."},
 	{PT_BOOL, EPF_NONE, PF_NA, "evt.failed", "'true' for events that returned an error status."},
@@ -1488,6 +1589,7 @@ const filtercheck_field_info sinsp_filter_check_event_fields[] =
 	{PT_BOOL, EPF_NONE, PF_NA, "evt.is_syslog", "'true' for events that are writes to /dev/log."},
 	{PT_UINT32, EPF_NONE, PF_DEC, "evt.count", "This filter field always returns 1 and can be used to count events from inside chisels."},
 	{PT_UINT64, EPF_FILTER_ONLY, PF_DEC, "evt.around", "Accepts the event if it's around the specified time interval. The syntax is evt.around[T]=D, where T is the value returned by %evt.rawtime for the event and D is a delta in milliseconds. For example, evt.around[1404996934793590564]=1000 will return the events with timestamp with one second before the timestamp and one second after it, for a total of two seconds of capture."},
+	{PT_CHARBUF, EPF_REQUIRES_ARGUMENT, PF_NA, "evt.abspath", "Absolute path calculated from dirfd and name during syscalls like renameat and symlinkat. Use 'evt.abspath.src' or 'evt.abspath.dst' for syscalls that support multiple paths."},
 };
 
 sinsp_filter_check_event::sinsp_filter_check_event()
@@ -1496,7 +1598,7 @@ sinsp_filter_check_event::sinsp_filter_check_event()
 	m_is_compare = false;
 	m_info.m_name = "evt";
 	m_info.m_fields = sinsp_filter_check_event_fields;
-	m_info.m_nfiedls = sizeof(sinsp_filter_check_event_fields) / sizeof(sinsp_filter_check_event_fields[0]);
+	m_info.m_nfields = sizeof(sinsp_filter_check_event_fields) / sizeof(sinsp_filter_check_event_fields[0]);
 	m_u64val = 0;
 }
 
@@ -1609,6 +1711,30 @@ int32_t sinsp_filter_check_event::parse_field_name(const char* str)
 		m_th_state_id = m_inspector->reserve_thread_memory(sizeof(uint16_t));
 		return sinsp_filter_check::parse_field_name(str);
 	}
+	else if(string(val, 0, sizeof("evt.abspath") - 1) == "evt.abspath")
+	{
+		m_field_id = TYPE_ABSPATH;
+		m_field = &m_info.m_fields[m_field_id];
+
+		if (val == "evt.abspath")
+		{
+			m_argid = 0;
+		}
+		else if (val == "evt.abspath.src")
+		{
+			m_argid = 1;
+		}
+		else if (val == "evt.abspath.dst")
+		{
+			m_argid = 2;
+		}
+		else
+		{
+			throw sinsp_exception("wrong syntax for evt.abspath");
+		}
+
+		return (int32_t)val.size() + 1;
+	}
 	else
 	{
 		return sinsp_filter_check::parse_field_name(str);
@@ -1631,6 +1757,7 @@ void sinsp_filter_check_event::parse_filter_value(const char* str, uint32_t len)
 	{
 		sinsp_evttables* einfo = m_inspector->get_event_info_tables();
 		const struct ppm_event_info* etable = einfo->m_event_info;
+		const struct ppm_syscall_desc* stable = einfo->m_syscall_info_table;
 		string stype(str, len);
 
 		for(uint32_t j = 0; j < PPM_EVENT_MAX; j++)
@@ -1641,6 +1768,14 @@ void sinsp_filter_check_event::parse_filter_value(const char* str, uint32_t len)
 			}
 		}
 
+		for(uint32_t j = 0; j < PPM_SC_MAX; j++)
+		{
+			if(stype == stable[j].name)
+			{
+				return sinsp_filter_check::parse_filter_value(str, len);
+			}
+		}
+
 		throw sinsp_exception("unknown event type " + stype);
 	}
 	else if(m_field_id == TYPE_AROUND)
@@ -1757,6 +1892,142 @@ uint8_t* extract_argraw(sinsp_evt *evt, OUT uint32_t* len, const char *argname)
 	}
 }
 
+uint8_t *sinsp_filter_check_event::extract_abspath(sinsp_evt *evt, OUT uint32_t *len)
+{
+	sinsp_evt_param *parinfo;
+	char *path;
+	uint32_t pathlen;
+	string spath;
+
+	if(evt->m_tinfo == NULL)
+	{
+		return NULL;
+	}
+
+	uint16_t etype = evt->get_type();
+
+	const char *dirfdarg = NULL, *patharg = NULL;
+	if (etype == PPME_SYSCALL_RENAMEAT_X)
+	{
+		if (m_argid == 1)
+		{
+			dirfdarg = "olddirfd";
+			patharg = "oldpath";
+		}
+		else if (m_argid == 2)
+		{
+			dirfdarg = "newdirfd";
+			patharg = "newpath";
+		}
+	}
+	else if (etype == PPME_SYSCALL_SYMLINKAT_X)
+	{
+		dirfdarg = "linkdirfd";
+		patharg = "linkpath";
+	}
+	else if (etype == PPME_SYSCALL_OPENAT_E)
+	{
+		dirfdarg = "dirfd";
+		patharg = "name";
+	}
+	else if (etype == PPME_SYSCALL_LINKAT_E)
+	{
+		if (m_argid == 1)
+		{
+			dirfdarg = "olddir";
+			patharg = "oldpath";
+		}
+		else if (m_argid == 2)
+		{
+			dirfdarg = "newdir";
+			patharg = "newpath";
+		}
+	}
+	else if (etype == PPME_SYSCALL_UNLINKAT_E)
+	{
+		dirfdarg = "dirfd";
+		patharg = "name";
+	}
+
+	if (!dirfdarg || !patharg)
+	{
+		return 0;
+	}
+
+	int dirfdargidx = -1, pathargidx = -1, idx = 0;
+	while (((dirfdargidx < 0) || (pathargidx < 0)) && (idx < (int) evt->get_num_params()))
+	{
+		const char *name = evt->get_param_name(idx);
+		if ((dirfdargidx < 0) && (strcmp(name, dirfdarg) == 0))
+		{
+			dirfdargidx = idx;
+		}
+		if ((pathargidx < 0) && (strcmp(name, patharg) == 0))
+		{
+			pathargidx = idx;
+		}
+		idx++;
+	}
+
+	if ((dirfdargidx < 0) || (pathargidx < 0))
+	{
+		return 0;
+	}
+
+	parinfo = evt->get_param(dirfdargidx);
+	ASSERT(parinfo->m_len == sizeof(int64_t));
+	int64_t dirfd = *(int64_t *)parinfo->m_val;
+
+	parinfo = evt->get_param(pathargidx);
+	path = parinfo->m_val;
+	pathlen = parinfo->m_len;
+
+	string sdir;
+
+	bool is_absolute = (path[0] == '/');
+	if(is_absolute)
+	{
+		//
+		// The path is absoulte.
+		// Some processes (e.g. irqbalance) actually do this: they pass an invalid fd and
+		// and bsolute path, and openat succeeds.
+		//
+		sdir = ".";
+	}
+	else if(dirfd == PPM_AT_FDCWD)
+	{
+		sdir = evt->m_tinfo->get_cwd();
+	}
+	else
+	{
+		evt->m_fdinfo = evt->m_tinfo->get_fd(dirfd);
+
+		if(evt->m_fdinfo == NULL)
+		{
+			ASSERT(false);
+			sdir = "<UNKNOWN>/";
+		}
+		else
+		{
+			if(evt->m_fdinfo->m_name[evt->m_fdinfo->m_name.length()] == '/')
+			{
+				sdir = evt->m_fdinfo->m_name;
+			}
+			else
+			{
+				sdir = evt->m_fdinfo->m_name + '/';
+			}
+		}
+	}
+
+	char fullname[SCAP_MAX_PATH_SIZE];
+	sinsp_utils::concatenate_paths(fullname, SCAP_MAX_PATH_SIZE, sdir.c_str(), (uint32_t)sdir.length(), path, pathlen);
+
+	m_strstorage = fullname;
+
+	return (uint8_t*)m_strstorage.c_str();
+}
+
 Json::Value sinsp_filter_check_event::extract_as_js(sinsp_evt *evt, OUT uint32_t* len)
 {
 	switch(m_field_id)
@@ -1839,24 +2110,9 @@ uint8_t* sinsp_filter_check_event::extract(sinsp_evt *evt, OUT uint32_t* len)
 		{
 			m_u64val = 0;
 
-			if(evt->get_direction() == SCAP_ED_IN)
-			{
-				if(evt->m_tinfo != NULL)
-				{
-					uint16_t* pt = (uint16_t*)evt->m_tinfo->get_private_state(m_th_state_id);
-					*pt = evt->get_type();
-				}
-
-				return (uint8_t*)&m_u64val;
-			}
-
 			if(evt->m_tinfo != NULL)
 			{
-				uint16_t* pt = (uint16_t*)evt->m_tinfo->get_private_state(m_th_state_id);
-				if(evt->m_tinfo->m_prevevent_ts && evt->get_type() == *pt + 1)
-				{
-					m_u64val = (evt->get_ts() - evt->m_tinfo->m_prevevent_ts);
-				}
+				m_u64val = evt->m_tinfo->m_latency;
 			}
 
 			return (uint8_t*)&m_u64val;
@@ -1866,30 +2122,17 @@ uint8_t* sinsp_filter_check_event::extract(sinsp_evt *evt, OUT uint32_t* len)
 		{
 			m_u64val = 0;
 
-			if(evt->get_direction() == SCAP_ED_IN)
-			{
-				if(evt->m_tinfo != NULL)
-				{
-					uint16_t* pt = (uint16_t*)evt->m_tinfo->get_private_state(m_th_state_id);
-					*pt = evt->get_type();
-				}
-
-				return (uint8_t*)&m_u64val;
-			}
-
 			if(evt->m_tinfo != NULL)
 			{
-				uint16_t* pt = (uint16_t*)evt->m_tinfo->get_private_state(m_th_state_id);
-				if(evt->m_tinfo->m_prevevent_ts && evt->get_type() == *pt + 1)
+				uint64_t lat = evt->m_tinfo->m_latency;
+
+				if(m_field_id == TYPE_LATENCY_S)
 				{
-					if(m_field_id == TYPE_LATENCY_S)
-					{
-						m_u64val = (evt->get_ts() - evt->m_tinfo->m_prevevent_ts) / 1000000000;
-					}
-					else
-					{
-						m_u64val = (evt->get_ts() - evt->m_tinfo->m_prevevent_ts) % 1000000000;
-					}
+					m_u64val = lat / 1000000000;
+				}
+				else
+				{
+					m_u64val = lat % 1000000000;
 				}
 			}
 
@@ -1997,13 +2240,12 @@ uint8_t* sinsp_filter_check_event::extract(sinsp_evt *evt, OUT uint32_t* len)
 		{
 			sinsp_fdinfo_t* fdinfo = evt->m_fdinfo;
 
-			if(fdinfo != NULL)
+			if(fdinfo != NULL && fdinfo->m_callbaks != NULL)
 			{
 				char* il;
-				vector<sinsp_protodecoder*>* cbacks = &(fdinfo->m_write_callbacks);
+				vector<sinsp_protodecoder*>* cbacks = &(fdinfo->m_callbaks->m_write_callbacks);
 
-				vector<sinsp_protodecoder*>::iterator it;
-				for(it = cbacks->begin(); it != cbacks->end(); ++it)
+				for(auto it = cbacks->begin(); it != cbacks->end(); ++it)
 				{
 					if((*it)->get_info_line(&il))
 					{
@@ -2070,6 +2312,34 @@ uint8_t* sinsp_filter_check_event::extract(sinsp_evt *evt, OUT uint32_t* len)
 
 			return (uint8_t*)argstr;
 		}
+	case TYPE_BUFLEN:
+		{
+			if(evt->get_direction() == SCAP_ED_OUT)
+			{
+				if(evt->get_category() & EC_IO_BASE)
+				{
+					if(evt->m_fdinfo)
+					{
+						sinsp_evt_param *parinfo;
+						int64_t retval;
+
+						//
+						// Extract the return value
+						//
+						parinfo = evt->get_param(0);
+						ASSERT(parinfo->m_len == sizeof(int64_t));
+						retval = *(int64_t *)parinfo->m_val;
+						
+						if(retval >= 0)
+						{
+							return (uint8_t*)parinfo->m_val;
+						}
+					}
+				}
+			}
+
+			return NULL;
+		}
 	case TYPE_RESRAW:
 		{
 			const sinsp_evt_param* pi = evt->get_param_value_raw("res");
@@ -2261,6 +2531,8 @@ uint8_t* sinsp_filter_check_event::extract(sinsp_evt *evt, OUT uint32_t* len)
 	case TYPE_COUNT:
 		m_u32val = 1;
 		return (uint8_t*)&m_u32val;
+	case TYPE_ABSPATH:
+		return extract_abspath(evt, len);
 	default:
 		ASSERT(false);
 		return NULL;
@@ -2335,7 +2607,7 @@ sinsp_filter_check_user::sinsp_filter_check_user()
 {
 	m_info.m_name = "user";
 	m_info.m_fields = sinsp_filter_check_user_fields;
-	m_info.m_nfiedls = sizeof(sinsp_filter_check_user_fields) / sizeof(sinsp_filter_check_user_fields[0]);
+	m_info.m_nfields = sizeof(sinsp_filter_check_user_fields) / sizeof(sinsp_filter_check_user_fields[0]);
 	m_info.m_flags = filter_check_info::FL_WORKS_ON_THREAD_TABLE;
 }
 
@@ -2360,7 +2632,6 @@ uint8_t* sinsp_filter_check_user::extract(sinsp_evt *evt, OUT uint32_t* len)
 
 		ASSERT(m_inspector != NULL);
 		const unordered_map<uint32_t, scap_userinfo*>* userlist = m_inspector->get_userlist();
-		ASSERT(userlist->size() != 0);
 
 		if(tinfo->m_uid == 0xffffffff)
 		{
@@ -2370,7 +2641,6 @@ uint8_t* sinsp_filter_check_user::extract(sinsp_evt *evt, OUT uint32_t* len)
 		it = userlist->find(tinfo->m_uid);
 		if(it == userlist->end())
 		{
-			ASSERT(false);
 			return NULL;
 		}
 
@@ -2409,7 +2679,7 @@ sinsp_filter_check_group::sinsp_filter_check_group()
 {
 	m_info.m_name = "group";
 	m_info.m_fields = sinsp_filter_check_group_fields;
-	m_info.m_nfiedls = sizeof(sinsp_filter_check_group_fields) / sizeof(sinsp_filter_check_group_fields[0]);
+	m_info.m_nfields = sizeof(sinsp_filter_check_group_fields) / sizeof(sinsp_filter_check_group_fields[0]);
 	m_info.m_flags = filter_check_info::FL_WORKS_ON_THREAD_TABLE;
 }
 
@@ -2525,7 +2795,7 @@ sinsp_filter_check_syslog::sinsp_filter_check_syslog()
 {
 	m_info.m_name = "syslog";
 	m_info.m_fields = sinsp_filter_check_syslog_fields;
-	m_info.m_nfiedls = sizeof(sinsp_filter_check_syslog_fields) / sizeof(sinsp_filter_check_syslog_fields[0]);
+	m_info.m_nfields = sizeof(sinsp_filter_check_syslog_fields) / sizeof(sinsp_filter_check_syslog_fields[0]);
 	m_decoder = NULL;
 }
 
@@ -2571,4 +2841,102 @@ uint8_t* sinsp_filter_check_syslog::extract(sinsp_evt *evt, OUT uint32_t* len)
 	}
 }
 
+///////////////////////////////////////////////////////////////////////////////
+// sinsp_filter_check_container implementation
+///////////////////////////////////////////////////////////////////////////////
+const filtercheck_field_info sinsp_filter_check_container_fields[] =
+{
+	{PT_CHARBUF, EPF_NONE, PF_NA, "container.id", "the container id."},
+	{PT_CHARBUF, EPF_NONE, PF_NA, "container.name", "the container name."},
+	{PT_CHARBUF, EPF_NONE, PF_NA, "container.image", "the container image."}
+};
+
+sinsp_filter_check_container::sinsp_filter_check_container()
+{
+	m_info.m_name = "container";
+	m_info.m_fields = sinsp_filter_check_container_fields;
+	m_info.m_nfields = sizeof(sinsp_filter_check_container_fields) / sizeof(sinsp_filter_check_container_fields[0]);
+	m_info.m_flags = filter_check_info::FL_WORKS_ON_THREAD_TABLE;
+}
+
+sinsp_filter_check* sinsp_filter_check_container::allocate_new()
+{
+	return (sinsp_filter_check*) new sinsp_filter_check_container();
+}
+
+uint8_t* sinsp_filter_check_container::extract(sinsp_evt *evt, OUT uint32_t* len)
+{
+	sinsp_threadinfo* tinfo = evt->get_thread_info();
+	if(tinfo == NULL)
+	{
+		return NULL;
+	}
+
+	switch(m_field_id)
+	{
+	case TYPE_CONTAINER_ID:
+		if(tinfo->m_container_id.empty())
+		{
+			m_tstr = "host";
+		}
+		else
+		{
+			m_tstr = tinfo->m_container_id;
+		}
+		
+		return (uint8_t*)m_tstr.c_str();
+	case TYPE_CONTAINER_NAME:
+		if(tinfo->m_container_id.empty())
+		{
+			m_tstr = "host";
+		}
+		else
+		{
+			sinsp_container_info container_info;
+			bool found = m_inspector->m_container_manager.get_container(tinfo->m_container_id, &container_info);
+			if(!found)
+			{
+				return NULL;
+			}
+
+			if(container_info.m_name.empty())
+			{
+				return NULL;
+			}
+
+			m_tstr = container_info.m_name;
+		}
+
+		return (uint8_t*)m_tstr.c_str();
+	case TYPE_CONTAINER_IMAGE:
+		if(tinfo->m_container_id.empty())
+		{
+			return NULL;
+		}
+		else
+		{
+			sinsp_container_info container_info;
+			bool found = m_inspector->m_container_manager.get_container(tinfo->m_container_id, &container_info);
+			if(!found)
+			{
+				return NULL;
+			}
+
+			if(container_info.m_image.empty())
+			{
+				return NULL;
+			}
+
+			m_tstr = container_info.m_image;
+		}
+
+		return (uint8_t*)m_tstr.c_str();
+	default:
+		ASSERT(false);
+		break;
+	}
+
+	return NULL;
+}
+
 #endif // HAS_FILTERING
