@@ -16,10 +16,8 @@ You should have received a copy of the GNU General Public License
 along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <fcntl.h>
 #ifdef _WIN32
+#define NOMINMAX
 #include <winsock2.h>
 #else
 #include <sys/socket.h>
@@ -29,6 +27,11 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 #include <unistd.h>
 #endif // _WIN32
 
+#include <stdio.h>
+#include <stdlib.h>
+#include <fcntl.h>
+#include <limits>
+
 #include "sinsp.h"
 #include "sinsp_int.h"
 #include "../../driver/ppm_ringbuffer.h"
@@ -52,9 +55,6 @@ sinsp_parser::sinsp_parser(sinsp *inspector) :
 	m_tmp_evt(m_inspector),
 	m_fd_listener(NULL)
 {
-#if defined(HAS_CAPTURE)
-	m_sysdig_pid = getpid();
-#endif
 }
 
 sinsp_parser::~sinsp_parser()
@@ -72,8 +72,8 @@ sinsp_parser::~sinsp_parser()
 ///////////////////////////////////////////////////////////////////////////////
 void sinsp_parser::process_event(sinsp_evt *evt)
 {
-	uint16_t etype = evt->get_type();
-	bool is_live = m_inspector->is_live();
+	uint16_t etype = evt->m_pevt->type;
+	bool is_live = m_inspector->m_islive;
 
 	//
 	// Cleanup the event-related state
@@ -86,10 +86,13 @@ void sinsp_parser::process_event(sinsp_evt *evt)
 #if defined(HAS_CAPTURE)
 	if(is_live && !m_inspector->is_debug_enabled())
 	{
-		if(evt->get_tid() == m_sysdig_pid && 
+		if(evt->get_tid() == m_inspector->m_sysdig_pid && 
 			etype != PPME_SCHEDSWITCH_1_E && 
 			etype != PPME_SCHEDSWITCH_6_E &&
-			m_sysdig_pid)
+			etype != PPME_DROP_E &&
+			etype != PPME_DROP_X &&
+			etype != PPME_SYSDIGEVENT_E &&
+			m_inspector->m_sysdig_pid)
 		{
 			evt->m_filtered_out = true;
 			return;
@@ -150,6 +153,11 @@ void sinsp_parser::process_event(sinsp_evt *evt)
 	case PPME_SYSCALL_PRLIMIT_E:
 	case PPME_SOCKET_SENDTO_E:
 	case PPME_SOCKET_SENDMSG_E:
+	case PPME_SYSCALL_SENDFILE_E:
+	case PPME_SYSCALL_SETRESUID_E:
+	case PPME_SYSCALL_SETRESGID_E:
+	case PPME_SYSCALL_SETUID_E:
+	case PPME_SYSCALL_SETGID_E:
 		store_event(evt);
 		break;
 	case PPME_SYSCALL_READ_X:
@@ -168,6 +176,9 @@ void sinsp_parser::process_event(sinsp_evt *evt)
 	case PPME_SYSCALL_PWRITEV_X:
 		parse_rw_exit(evt);
 		break;
+	case PPME_SYSCALL_SENDFILE_X:
+		parse_sendfile_exit(evt);
+		break;
 	case PPME_SYSCALL_OPEN_X:
 	case PPME_SYSCALL_CREAT_X:
 	case PPME_SYSCALL_OPENAT_X:
@@ -178,16 +189,27 @@ void sinsp_parser::process_event(sinsp_evt *evt)
 	case PPME_SYSCALL_EPOLLWAIT_E:
 		parse_select_poll_epollwait_enter(evt);
 		break;
-	case PPME_CLONE_11_X:
-	case PPME_CLONE_16_X:
+	case PPME_SYSCALL_CLONE_11_X:
+	case PPME_SYSCALL_CLONE_16_X:
+	case PPME_SYSCALL_CLONE_17_X:
+	case PPME_SYSCALL_CLONE_20_X:
+	case PPME_SYSCALL_FORK_X:
+	case PPME_SYSCALL_FORK_17_X:
+	case PPME_SYSCALL_FORK_20_X:
+	case PPME_SYSCALL_VFORK_X:
+	case PPME_SYSCALL_VFORK_17_X:
+	case PPME_SYSCALL_VFORK_20_X:
 		parse_clone_exit(evt);
 		break;
 	case PPME_SYSCALL_EXECVE_8_X:
 	case PPME_SYSCALL_EXECVE_13_X:
 	case PPME_SYSCALL_EXECVE_14_X:
+	case PPME_SYSCALL_EXECVE_15_X:
+	case PPME_SYSCALL_EXECVE_16_X:
 		parse_execve_exit(evt);
 		break;
 	case PPME_PROCEXIT_E:
+	case PPME_PROCEXIT_1_E:
 		parse_thread_exit(evt);
 		break;
 	case PPME_SYSCALL_PIPE_X:
@@ -264,6 +286,22 @@ void sinsp_parser::process_event(sinsp_evt *evt)
 	case PPME_SYSCALL_MMAP2_X:
 	case PPME_SYSCALL_MUNMAP_X:
 		parse_brk_munmap_mmap_exit(evt);
+		break;
+	case PPME_SYSCALL_SETRESUID_X:
+		parse_setresuid_exit(evt);
+		break;
+	case PPME_SYSCALL_SETRESGID_X:
+		parse_setresgid_exit(evt);
+		break;
+	case PPME_SYSCALL_SETUID_X:
+		parse_setuid_exit(evt);
+		break;
+	case PPME_SYSCALL_SETGID_X:
+		parse_setgid_exit(evt);
+		break;
+	case PPME_CONTAINER_E:
+		parse_container_evt(evt);
+		break;
 	default:
 		break;
 	}
@@ -327,6 +365,7 @@ bool sinsp_parser::reset(sinsp_evt *evt)
 	//
 	if(eflags & EF_SKIPPARSERESET)
 	{
+		evt->m_tinfo = NULL;
 		return false;
 	}
 
@@ -339,8 +378,16 @@ bool sinsp_parser::reset(sinsp_evt *evt)
 	// (many kernel thread), we don't look for /proc
 	//
 	bool query_os;
-	if(etype == PPME_CLONE_11_X ||
-		etype == PPME_CLONE_16_X ||
+	if(etype == PPME_SYSCALL_CLONE_11_X ||
+		etype == PPME_SYSCALL_CLONE_16_X ||
+		etype == PPME_SYSCALL_CLONE_17_X ||
+		etype == PPME_SYSCALL_CLONE_20_X ||
+		etype == PPME_SYSCALL_FORK_X ||
+		etype == PPME_SYSCALL_FORK_17_X ||
+		etype == PPME_SYSCALL_FORK_20_X ||
+		etype == PPME_SYSCALL_VFORK_X ||
+		etype == PPME_SYSCALL_VFORK_17_X ||
+		etype == PPME_SYSCALL_VFORK_20_X ||
 		etype == PPME_SCHEDSWITCH_6_E)
 	{
 		query_os = false;
@@ -350,7 +397,7 @@ bool sinsp_parser::reset(sinsp_evt *evt)
 		query_os = true;
 	}
 
-	evt->m_tinfo = evt->get_thread_info(query_os);
+	evt->m_tinfo = m_inspector->get_thread(evt->m_pevt->tid, query_os, false);
 
 	if(etype == PPME_SCHEDSWITCH_6_E)
 	{
@@ -359,8 +406,16 @@ bool sinsp_parser::reset(sinsp_evt *evt)
 
 	if(!evt->m_tinfo)
 	{
-		if(etype == PPME_CLONE_11_X ||
-			etype == PPME_CLONE_16_X)
+		if(etype == PPME_SYSCALL_CLONE_11_X ||
+			etype == PPME_SYSCALL_CLONE_16_X ||
+			etype == PPME_SYSCALL_CLONE_17_X ||
+			etype == PPME_SYSCALL_CLONE_20_X ||
+			etype == PPME_SYSCALL_FORK_X ||
+			etype == PPME_SYSCALL_FORK_17_X ||
+			etype == PPME_SYSCALL_FORK_20_X ||
+			etype == PPME_SYSCALL_VFORK_X ||
+			etype == PPME_SYSCALL_VFORK_17_X ||
+			etype == PPME_SYSCALL_VFORK_20_X)
 		{
 #ifdef GATHER_INTERNAL_STATS
 			m_inspector->m_thread_manager->m_failed_lookups->decrement();
@@ -374,6 +429,11 @@ bool sinsp_parser::reset(sinsp_evt *evt)
 		return false;
 	}
 
+	if(query_os)
+	{
+		evt->m_tinfo->m_flags |= PPM_CL_ACTIVE;
+	}
+
 	if(PPME_IS_ENTER(etype))
 	{
 		evt->m_tinfo->m_lastevent_fd = -1;
@@ -399,22 +459,24 @@ bool sinsp_parser::reset(sinsp_evt *evt)
 	}
 	else
 	{
+		sinsp_threadinfo* tinfo = evt->m_tinfo;
+
 		//
 		// event latency
 		//
-		if(evt->m_tinfo->m_last_latency_entertime != 0)
+		if(tinfo->m_last_latency_entertime != 0)
 		{
-			evt->m_tinfo->m_latency = evt->get_ts() - evt->m_tinfo->m_last_latency_entertime;
-			ASSERT((int64_t)evt->m_tinfo->m_latency >= 0);
+			tinfo->m_latency = evt->get_ts() - tinfo->m_last_latency_entertime;
+			ASSERT((int64_t)tinfo->m_latency >= 0);
 		}
 
-		if(etype == evt->m_tinfo->m_lastevent_type + 1)
+		if(etype == tinfo->m_lastevent_type + 1)
 		{
-			evt->m_tinfo->set_lastevent_data_validity(true);
+			tinfo->set_lastevent_data_validity(true);
 		}
 		else
 		{
-			evt->m_tinfo->set_lastevent_data_validity(false);
+			tinfo->set_lastevent_data_validity(false);
 			return false;
 		}
 
@@ -442,7 +504,7 @@ bool sinsp_parser::reset(sinsp_evt *evt)
 		//
 		if(eflags & EF_USES_FD)
 		{
-			evt->m_fdinfo = evt->m_tinfo->get_fd(evt->m_tinfo->m_lastevent_fd);
+			evt->m_fdinfo = tinfo->get_fd(tinfo->m_lastevent_fd);
 
 			if(evt->m_fdinfo == NULL)
 			{
@@ -465,14 +527,14 @@ bool sinsp_parser::reset(sinsp_evt *evt)
 
 				evt->m_fdinfo->m_flags &= ~sinsp_fdinfo_t::FLAGS_CLOSE_CANCELED;
 				eparams.m_fd = CANCELED_FD_NUMBER;
-				eparams.m_fdinfo = evt->m_tinfo->get_fd(CANCELED_FD_NUMBER);
+				eparams.m_fdinfo = tinfo->get_fd(CANCELED_FD_NUMBER);
 
 				//
 				// Remove the fd from the different tables
 				//
 				eparams.m_remove_from_table = true;
 				eparams.m_inspector = m_inspector;
-				eparams.m_tinfo = evt->m_tinfo;
+				eparams.m_tinfo = tinfo;
 				eparams.m_ts = evt->get_ts();
 
 				erase_fd(&eparams);
@@ -602,10 +664,13 @@ void sinsp_parser::parse_clone_exit(sinsp_evt *evt)
 	sinsp_evt_param *parinfo;
 	int64_t tid = evt->get_tid();
 	int64_t childtid;
-	unordered_map<int64_t, sinsp_threadinfo>::iterator it;
 	bool is_inverted_clone = false; // true if clone() in the child returns before the one in the parent
 	bool tid_collision = false;
 	bool valid_parent = true;
+	bool in_container = false;
+	int64_t vtid = tid;
+	int64_t vpid = -1;
+	uint16_t etype = evt->get_type();
 
 	//
 	// Validate the return value and get the child tid
@@ -614,6 +679,32 @@ void sinsp_parser::parse_clone_exit(sinsp_evt *evt)
 	ASSERT(parinfo->m_len == sizeof(int64_t));
 	childtid = *(int64_t *)parinfo->m_val;
 
+	switch(evt->get_type())
+	{
+	case PPME_SYSCALL_CLONE_11_X:
+		parinfo = evt->get_param(8);
+		break;
+	case PPME_SYSCALL_CLONE_16_X:
+	case PPME_SYSCALL_FORK_X:
+	case PPME_SYSCALL_VFORK_X:
+		parinfo = evt->get_param(13);
+		break;
+	case PPME_SYSCALL_CLONE_17_X:
+	case PPME_SYSCALL_FORK_17_X:
+	case PPME_SYSCALL_VFORK_17_X:
+		parinfo = evt->get_param(14);
+		break;
+	case PPME_SYSCALL_CLONE_20_X:
+	case PPME_SYSCALL_FORK_20_X:
+	case PPME_SYSCALL_VFORK_20_X:
+		parinfo = evt->get_param(15);
+		break;
+	default:
+		ASSERT(false);
+	}
+	ASSERT(parinfo->m_len == sizeof(int32_t));
+	uint32_t flags = *(int32_t *)parinfo->m_val;
+
 	if(childtid < 0)
 	{
 		//
@@ -621,7 +712,41 @@ void sinsp_parser::parse_clone_exit(sinsp_evt *evt)
 		//
 		return;
 	}
-	else if(childtid == 0)
+	
+	//
+	// Get the vtid to check if the clone is within a container
+	//
+	switch(etype)
+	{
+	case PPME_SYSCALL_CLONE_11_X:
+	case PPME_SYSCALL_CLONE_16_X:
+	case PPME_SYSCALL_CLONE_17_X:
+	case PPME_SYSCALL_FORK_X:
+	case PPME_SYSCALL_FORK_17_X:
+	case PPME_SYSCALL_VFORK_X:
+	case PPME_SYSCALL_VFORK_17_X:
+		break;
+	case PPME_SYSCALL_CLONE_20_X:
+	case PPME_SYSCALL_FORK_20_X:
+	case PPME_SYSCALL_VFORK_20_X:
+		parinfo = evt->get_param(18);
+		ASSERT(parinfo->m_len == sizeof(int64_t));
+		vtid = *(int64_t *)parinfo->m_val;
+
+		parinfo = evt->get_param(19);
+		ASSERT(parinfo->m_len == sizeof(int64_t));
+		vpid = *(int64_t *)parinfo->m_val;
+		break;
+	default:
+		ASSERT(false);
+	}
+
+	if(tid != vtid)
+	{
+		in_container = true;
+	}
+
+	if(childtid == 0)
 	{
 		//
 		// clone() returns 0 in the child.
@@ -634,7 +759,12 @@ void sinsp_parser::parse_clone_exit(sinsp_evt *evt)
 			// This happens if
 			//  - clone() returns in the child before than in the parent.
 			//  - we dropped the clone exit event in the parent.
+			//  - clone was executed in a container
 			// In both cases, we create the thread entry here
+			//
+			// XXX: inverted_clone flag should be useless for containers
+			// since just the child's clone is allowed to create a thread
+			//
 			is_inverted_clone = true;
 
 			//
@@ -643,22 +773,8 @@ void sinsp_parser::parse_clone_exit(sinsp_evt *evt)
 			childtid = tid;
 
 			//
-			// Get the flags, and check if this is a process or a new thread
+			// Check if this is a process or a new thread
 			//
-			switch(evt->get_type())
-			{
-				case PPME_CLONE_11_X:
-					parinfo = evt->get_param(8);
-					break;
-				case PPME_CLONE_16_X:
-					parinfo = evt->get_param(13);
-					break;
-				default:
-					ASSERT(false);
-			}
-			ASSERT(parinfo->m_len == sizeof(int32_t));
-			uint32_t flags = *(int32_t *)parinfo->m_val;
-
 			if(flags & PPM_CL_CLONE_THREAD)
 			{
 				//
@@ -683,6 +799,30 @@ void sinsp_parser::parse_clone_exit(sinsp_evt *evt)
 			//
 		}
 		else
+		{
+			//
+			// We are in the child's clone. If we are in a container, make
+			// sure the vtid/vpid are reflected because the father was maybe
+			// running outside the container so created the child thread without
+			// knowing the internal vtid/vpid
+			//
+			if(in_container)
+			{
+				evt->m_tinfo->m_vtid = vtid;
+				evt->m_tinfo->m_vpid = vpid;
+			}
+
+			return;
+		}
+	}
+	else
+	{
+		//
+		// We are in the father. If the father is running in a container,
+		// don't create the child process but wait until we see child, because
+		// the father just sees the internal tid of the child
+		//
+		if(in_container)
 		{
 			return;
 		}
@@ -725,7 +865,6 @@ void sinsp_parser::parse_clone_exit(sinsp_evt *evt)
 		}
 		else
 		{
-			ASSERT(evt->get_num() < 10000);
 			m_inspector->remove_thread(childtid, true);
 			tid_collision = true;
 		}
@@ -791,8 +930,29 @@ void sinsp_parser::parse_clone_exit(sinsp_evt *evt)
 			// Parent not found in proc, use the event data
 			//
 			parinfo = evt->get_param(1);
-			tinfo.m_comm = (char*)parinfo->m_val;
-			tinfo.m_exe = tinfo.m_comm;
+			tinfo.m_exe = (char*)parinfo->m_val;
+
+			switch(etype)
+			{
+			case PPME_SYSCALL_CLONE_11_X:
+			case PPME_SYSCALL_CLONE_16_X:
+			case PPME_SYSCALL_FORK_X:
+			case PPME_SYSCALL_VFORK_X:
+				tinfo.m_comm = tinfo.m_exe;
+				break;
+			case PPME_SYSCALL_CLONE_17_X:
+			case PPME_SYSCALL_CLONE_20_X:
+			case PPME_SYSCALL_FORK_17_X:
+			case PPME_SYSCALL_FORK_20_X:
+			case PPME_SYSCALL_VFORK_17_X:
+			case PPME_SYSCALL_VFORK_20_X:
+				parinfo = evt->get_param(13);
+				tinfo.m_comm = parinfo->m_val;
+				break;
+			default:
+				ASSERT(false);
+			}
+
 			parinfo = evt->get_param(2);
 			tinfo.set_args(parinfo->m_val, parinfo->m_len);
 
@@ -811,19 +971,7 @@ void sinsp_parser::parse_clone_exit(sinsp_evt *evt)
 	tinfo.m_pid = *(int64_t *)parinfo->m_val;
 
 	// Get the flags, and check if this is a thread or a new thread
-	switch(evt->get_type())
-	{
-		case PPME_CLONE_11_X:
-			parinfo = evt->get_param(8);
-			break;
-		case PPME_CLONE_16_X:
-			parinfo = evt->get_param(13);
-			break;
-		default:
-			ASSERT(false);
-	}
-	ASSERT(parinfo->m_len == sizeof(int32_t));
-	tinfo.m_flags = *(int32_t *)parinfo->m_val;
+	tinfo.m_flags = flags;
 
 	//
 	// If clone()'s PPM_CL_CLONE_THREAD is not set it means that a new
@@ -862,6 +1010,35 @@ void sinsp_parser::parse_clone_exit(sinsp_evt *evt)
 		tinfo.m_flags |= PPM_CL_CLONE_INVERTED;
 	}
 
+	// Copy the command name
+	parinfo = evt->get_param(1);
+	tinfo.m_exe = (char*)parinfo->m_val;
+
+	switch(etype)
+	{
+	case PPME_SYSCALL_CLONE_11_X:
+	case PPME_SYSCALL_CLONE_16_X:
+	case PPME_SYSCALL_FORK_X:
+	case PPME_SYSCALL_VFORK_X:
+		tinfo.m_comm = tinfo.m_exe;
+		break;
+	case PPME_SYSCALL_CLONE_17_X:
+	case PPME_SYSCALL_CLONE_20_X:
+	case PPME_SYSCALL_FORK_17_X:
+	case PPME_SYSCALL_FORK_20_X:
+	case PPME_SYSCALL_VFORK_17_X:
+	case PPME_SYSCALL_VFORK_20_X:
+		parinfo = evt->get_param(13);
+		tinfo.m_comm = parinfo->m_val;
+		break;
+	default:
+		ASSERT(false);
+	}
+
+	// Get the command arguments
+	parinfo = evt->get_param(2);
+	tinfo.set_args(parinfo->m_val, parinfo->m_len);
+
 	// Copy the working directory
 	parinfo = evt->get_param(6);
 	tinfo.set_cwd(parinfo->m_val, parinfo->m_len);
@@ -871,8 +1048,19 @@ void sinsp_parser::parse_clone_exit(sinsp_evt *evt)
 	ASSERT(parinfo->m_len == sizeof(int64_t));
 	tinfo.m_fdlimit = *(int64_t *)parinfo->m_val;
 
-	if(evt->get_type() == PPME_CLONE_16_X)
+	switch(etype)
 	{
+	case PPME_SYSCALL_CLONE_11_X:
+		break;
+	case PPME_SYSCALL_CLONE_16_X:
+	case PPME_SYSCALL_CLONE_17_X:
+	case PPME_SYSCALL_CLONE_20_X:
+	case PPME_SYSCALL_FORK_X:
+	case PPME_SYSCALL_FORK_17_X:
+	case PPME_SYSCALL_FORK_20_X:
+	case PPME_SYSCALL_VFORK_X:
+	case PPME_SYSCALL_VFORK_17_X:
+	case PPME_SYSCALL_VFORK_20_X:
 		// Get the pgflt_maj
 		parinfo = evt->get_param(8);
 		ASSERT(parinfo->m_len == sizeof(uint64_t));
@@ -897,38 +1085,97 @@ void sinsp_parser::parse_clone_exit(sinsp_evt *evt)
 		parinfo = evt->get_param(12);
 		ASSERT(parinfo->m_len == sizeof(uint32_t));
 		tinfo.m_vmswap_kb = *(uint32_t *)parinfo->m_val;
+		break;
+	default:
+		ASSERT(false);
 	}
 
 	// Copy the uid
-	switch(evt->get_type())
+	switch(etype)
 	{
-		case PPME_CLONE_11_X:
-			parinfo = evt->get_param(9);
-			break;
-		case PPME_CLONE_16_X:
-			parinfo = evt->get_param(14);
-			break;
-		default:
-			ASSERT(false);
+	case PPME_SYSCALL_CLONE_11_X:
+		parinfo = evt->get_param(9);
+		break;
+	case PPME_SYSCALL_CLONE_16_X:
+	case PPME_SYSCALL_FORK_X:
+	case PPME_SYSCALL_VFORK_X:
+		parinfo = evt->get_param(14);
+		break;
+	case PPME_SYSCALL_CLONE_17_X:
+	case PPME_SYSCALL_FORK_17_X:
+	case PPME_SYSCALL_VFORK_17_X:
+		parinfo = evt->get_param(15);
+		break;
+	case PPME_SYSCALL_CLONE_20_X:
+	case PPME_SYSCALL_FORK_20_X:
+	case PPME_SYSCALL_VFORK_20_X:
+		parinfo = evt->get_param(16);
+		break;
+	default:
+		ASSERT(false);
 	}
 	ASSERT(parinfo->m_len == sizeof(int32_t));
 	tinfo.m_uid = *(int32_t *)parinfo->m_val;
 
 	// Copy the uid
-	switch(evt->get_type())
+	switch(etype)
 	{
-		case PPME_CLONE_11_X:
-			parinfo = evt->get_param(10);
-			break;
-		case PPME_CLONE_16_X:
-			parinfo = evt->get_param(15);
-			break;
-		default:
-			ASSERT(false);
+	case PPME_SYSCALL_CLONE_11_X:
+		parinfo = evt->get_param(10);
+		break;
+	case PPME_SYSCALL_CLONE_16_X:
+	case PPME_SYSCALL_FORK_X:
+	case PPME_SYSCALL_VFORK_X:
+		parinfo = evt->get_param(15);
+		break;
+	case PPME_SYSCALL_CLONE_17_X:
+	case PPME_SYSCALL_FORK_17_X:
+	case PPME_SYSCALL_VFORK_17_X:
+		parinfo = evt->get_param(16);
+		break;
+	case PPME_SYSCALL_CLONE_20_X:
+	case PPME_SYSCALL_FORK_20_X:
+	case PPME_SYSCALL_VFORK_20_X:
+		parinfo = evt->get_param(17);
+		break;
+	default:
+		ASSERT(false);
 	}
 	ASSERT(parinfo->m_len == sizeof(int32_t));
 	tinfo.m_gid = *(int32_t *)parinfo->m_val;
 
+	//
+	// If we're in a container, vtid and vpid are
+	// initialized to the values coming from the event,
+	// otherwise they are just set to tid and pid. We can't
+	// use the event in that case because in a non-container
+	// case also the clone exit from the father can create a
+	// child process, and it doesn't have the right vtid and vpid
+	// values
+	//
+	if(in_container)
+	{
+		tinfo.m_vtid = vtid;
+		tinfo.m_vpid = vpid;
+	}
+	else
+	{
+		tinfo.m_vtid = tinfo.m_tid;
+		tinfo.m_vpid = tinfo.m_vpid;
+	}
+
+	//
+	// Set cgroups and heuristically detect container id
+	//
+	switch(etype)
+	{
+		case PPME_SYSCALL_CLONE_20_X:
+			parinfo = evt->get_param(14);
+			tinfo.set_cgroups(parinfo->m_val, parinfo->m_len);
+			m_inspector->m_container_manager.resolve_container_from_cgroups(tinfo.m_cgroups, m_inspector->m_islive, &tinfo.m_container_id);
+			break;
+	}
+
 	//
 	// Initilaize the thread clone time
 	//
@@ -947,9 +1194,14 @@ void sinsp_parser::parse_clone_exit(sinsp_evt *evt)
 	if(tid_collision)
 	{
 		reset(evt);
+#ifdef HAS_ANALYZER
+		m_inspector->m_tid_collisions.push_back(tinfo.m_tid);
+#endif
+#ifdef _DEBUG
 		g_logger.format(sinsp_logger::SEV_INFO, 
 			"tid collision for %" PRIu64 "(%s)", 
 			tinfo.m_tid, tinfo.m_comm.c_str());
+#endif
 	}
 
 	return;
@@ -959,6 +1211,7 @@ void sinsp_parser::parse_execve_exit(sinsp_evt *evt)
 {
 	sinsp_evt_param *parinfo;
 	int64_t retval;
+	uint16_t etype = evt->get_type();
 
 	// Validate the return value
 	parinfo = evt->get_param(0);
@@ -985,21 +1238,28 @@ void sinsp_parser::parse_execve_exit(sinsp_evt *evt)
 		return;
 	}
 
-	string prev_comm(evt->m_tinfo->m_comm);
-	string prev_exe(evt->m_tinfo->m_exe);
-
-	// Get the command name
+	// Get the exe
 	parinfo = evt->get_param(1);
-	string tmps = parinfo->m_val;
-	tmps = tmps.substr(tmps.rfind("/") + 1);
-	evt->m_tinfo->m_comm = tmps;
-
-	//
-	// XXX We should retrieve the full executable name from the arguments that execve receives in the kernel,
-	// but for the moment we don't do it, so we just copy the command name into the exe string
-	//
 	evt->m_tinfo->m_exe = parinfo->m_val;
 
+	switch(etype)
+	{
+	case PPME_SYSCALL_EXECVE_8_X:
+	case PPME_SYSCALL_EXECVE_13_X:
+	case PPME_SYSCALL_EXECVE_14_X:
+		// Old trace files didn't have comm, so just set it to exe
+		evt->m_tinfo->m_comm = evt->m_tinfo->m_exe;
+		break;
+	case PPME_SYSCALL_EXECVE_15_X:
+	case PPME_SYSCALL_EXECVE_16_X:
+		// Get the comm
+		parinfo = evt->get_param(13);
+		evt->m_tinfo->m_comm = parinfo->m_val;
+		break;
+	default:
+		ASSERT(false);
+	}
+
 	// Get the command arguments
 	parinfo = evt->get_param(2);
 	evt->m_tinfo->set_args(parinfo->m_val, parinfo->m_len);
@@ -1018,9 +1278,14 @@ void sinsp_parser::parse_execve_exit(sinsp_evt *evt)
 	ASSERT(parinfo->m_len == sizeof(int64_t));
 	evt->m_tinfo->m_fdlimit = *(int64_t *)parinfo->m_val;
 
-	if(evt->get_type() == PPME_SYSCALL_EXECVE_13_X ||
-		evt->get_type() == PPME_SYSCALL_EXECVE_14_X)
+	switch(etype)
 	{
+	case PPME_SYSCALL_EXECVE_8_X:
+		break;
+	case PPME_SYSCALL_EXECVE_13_X:
+	case PPME_SYSCALL_EXECVE_14_X:
+	case PPME_SYSCALL_EXECVE_15_X:
+	case PPME_SYSCALL_EXECVE_16_X:
 		// Get the pgflt_maj
 		parinfo = evt->get_param(8);
 		ASSERT(parinfo->m_len == sizeof(uint64_t));
@@ -1045,13 +1310,43 @@ void sinsp_parser::parse_execve_exit(sinsp_evt *evt)
 		parinfo = evt->get_param(12);
 		ASSERT(parinfo->m_len == sizeof(uint32_t));
 		evt->m_tinfo->m_vmswap_kb = *(uint32_t *)parinfo->m_val;
+		break;
+	default:
+		ASSERT(false);
+	}
 
-		if(evt->get_type() == PPME_SYSCALL_EXECVE_14_X)
+	switch(etype)
+	{
+	case PPME_SYSCALL_EXECVE_8_X:
+	case PPME_SYSCALL_EXECVE_13_X:
+		break;
+	case PPME_SYSCALL_EXECVE_14_X:
+		// Get the environment
+		parinfo = evt->get_param(13);
+		evt->m_tinfo->set_env(parinfo->m_val, parinfo->m_len);
+		break;
+	case PPME_SYSCALL_EXECVE_15_X:
+		// Get the environment
+		parinfo = evt->get_param(14);
+		evt->m_tinfo->set_env(parinfo->m_val, parinfo->m_len);
+		break;
+	case PPME_SYSCALL_EXECVE_16_X:
+		// Get the environment
+		parinfo = evt->get_param(15);
+		evt->m_tinfo->set_env(parinfo->m_val, parinfo->m_len);
+
+		//
+		// Set cgroups and heuristically detect container id
+		//
+		parinfo = evt->get_param(14);
+		evt->m_tinfo->set_cgroups(parinfo->m_val, parinfo->m_len);
+		if(evt->m_tinfo->m_container_id.empty())
 		{
-			// Get the environment
-			parinfo = evt->get_param(13);
-			evt->m_tinfo->set_env(parinfo->m_val, parinfo->m_len);
+			m_inspector->m_container_manager.resolve_container_from_cgroups(evt->m_tinfo->m_cgroups, m_inspector->m_islive, &evt->m_tinfo->m_container_id);
 		}
+		break;
+	default:
+		ASSERT(false);
 	}
 
 	//
@@ -1065,7 +1360,7 @@ void sinsp_parser::parse_execve_exit(sinsp_evt *evt)
 	// Clear the flags for this thread, making sure to propagate the inverted flag
 	//
 	bool inverted = ((evt->m_tinfo->m_flags & PPM_CL_CLONE_INVERTED) != 0);
-	evt->m_tinfo->m_flags = 0;
+	evt->m_tinfo->m_flags = PPM_CL_ACTIVE;
 	if(inverted)
 	{
 		evt->m_tinfo->m_flags |= PPM_CL_CLONE_INVERTED;
@@ -1077,19 +1372,9 @@ void sinsp_parser::parse_execve_exit(sinsp_evt *evt)
 	evt->m_tinfo->m_flags |= PPM_CL_NAME_CHANGED;
 
 	//
-	// execve potentially breaks the program chain, and so we need to reflect it in our parents program count.
+	// Recompute the program hash
 	//
-	if((prev_comm != evt->m_tinfo->m_comm) || (prev_exe != evt->m_tinfo->m_exe))
-	{
-		if(evt->m_tinfo->m_progid != -1LL)
-		{
-			m_inspector->m_thread_manager->decrement_program_childcount(evt->m_tinfo);
-		}
-		else
-		{
-			m_inspector->m_thread_manager->increment_program_childcount(evt->m_tinfo, 0, 0);
-		}
-	}
+	evt->m_tinfo->compute_program_hash();
 
 #ifdef HAS_ANALYZER
 	evt->m_tinfo->m_ainfo->clear_role_flags();
@@ -1658,7 +1943,6 @@ void sinsp_parser::parse_accept_exit(sinsp_evt *evt)
 		m_fd_listener->on_accept(evt, fd, packed_data, &fdi);
 	}
 
-
 	//
 	// Mark this fd as a server
 	//
@@ -1668,7 +1952,6 @@ void sinsp_parser::parse_accept_exit(sinsp_evt *evt)
 	// Add the entry to the table
 	//
 	evt->m_fdinfo = evt->m_tinfo->add_fd(fd, &fdi);
-	ASSERT(evt->m_fdinfo != NULL);
 }
 
 void sinsp_parser::parse_close_enter(sinsp_evt *evt)
@@ -1764,8 +2047,6 @@ void sinsp_parser::parse_close_exit(sinsp_evt *evt)
 			eparams.m_fdinfo = evt->m_fdinfo;
 		}
 
-		//m_inspector->push_fdop(tid, evt->m_fdinfo, sinsp_fdop(fd, evt->get_type()));
-
 		//
 		// Remove the fd from the different tables
 		//
@@ -2210,18 +2491,21 @@ void sinsp_parser::parse_rw_exit(sinsp_evt *evt)
 			//
 			if(m_fd_listener)
 			{
-				m_fd_listener->on_read(evt, tid, evt->m_tinfo->m_lastevent_fd, data, (uint32_t)retval, datalen);
+				m_fd_listener->on_read(evt, tid, evt->m_tinfo->m_lastevent_fd, evt->m_fdinfo, 
+					data, (uint32_t)retval, datalen);
 			}
 
 			//
 			// Call the protocol decoder callbacks associated to this event
 			//
-			vector<sinsp_protodecoder*>* cbacks = &(evt->m_fdinfo->m_read_callbacks);
-
-			vector<sinsp_protodecoder*>::iterator it;
-			for(it = cbacks->begin(); it != cbacks->end(); ++it)
+			if(evt->m_fdinfo->m_callbaks)
 			{
-				(*it)->on_read(evt, data, datalen);
+				vector<sinsp_protodecoder*>* cbacks = &(evt->m_fdinfo->m_callbaks->m_read_callbacks);
+
+				for(auto it = cbacks->begin(); it != cbacks->end(); ++it)
+				{
+					(*it)->on_read(evt, data, datalen);
+				}
 			}
 		}
 		else
@@ -2293,23 +2577,73 @@ void sinsp_parser::parse_rw_exit(sinsp_evt *evt)
 			//
 			if(m_fd_listener)
 			{
-				m_fd_listener->on_write(evt, tid, evt->m_tinfo->m_lastevent_fd, data, (uint32_t)retval, datalen);
+				m_fd_listener->on_write(evt, tid, evt->m_tinfo->m_lastevent_fd, evt->m_fdinfo,
+					data, (uint32_t)retval, datalen);
 			}
 
 			//
 			// Call the protocol decoder callbacks associated to this event
 			//
-			vector<sinsp_protodecoder*>* cbacks = &(evt->m_fdinfo->m_write_callbacks);
-
-			vector<sinsp_protodecoder*>::iterator it;
-			for(it = cbacks->begin(); it != cbacks->end(); ++it)
+			if(evt->m_fdinfo->m_callbaks)
 			{
-				(*it)->on_write(evt, data, datalen);
+				vector<sinsp_protodecoder*>* cbacks = &(evt->m_fdinfo->m_callbaks->m_write_callbacks);
+
+				for(auto it = cbacks->begin(); it != cbacks->end(); ++it)
+				{
+					(*it)->on_write(evt, data, datalen);
+				}
 			}
 		}
 	}
 }
 
+void sinsp_parser::parse_sendfile_exit(sinsp_evt *evt)
+{
+	sinsp_evt_param *parinfo;
+	int64_t retval;
+
+	if(!evt->m_fdinfo)
+	{
+		return;
+	}
+
+	//
+	// Extract the return value
+	//
+	parinfo = evt->get_param(0);
+	ASSERT(parinfo->m_len == sizeof(int64_t));
+	retval = *(int64_t *)parinfo->m_val;
+
+	//
+	// If the operation was successful, validate that the fd exists
+	//
+	if(retval >= 0)
+	{
+		sinsp_evt *enter_evt = &m_tmp_evt;
+		int64_t fdin;
+
+		if(!retrieve_enter_event(enter_evt, evt))
+		{
+			return;
+		}
+
+		//
+		// Extract the in FD
+		//
+		parinfo = enter_evt->get_param(1);
+		ASSERT(parinfo->m_len == sizeof(int64_t));
+		fdin = *(int64_t *)parinfo->m_val;
+
+		//
+		// If there's an fd listener, call it now
+		//
+		if(m_fd_listener)
+		{
+			m_fd_listener->on_sendfile(evt, fdin, (uint32_t)retval);
+		}
+	}
+}
+
 void sinsp_parser::parse_eventfd_exit(sinsp_evt *evt)
 {
 	sinsp_evt_param *parinfo;
@@ -2525,10 +2859,27 @@ void sinsp_parser::parse_dup_exit(sinsp_evt *evt)
 			return;
 		}
 
+		//
+		// If the old FD is in the table, remove it properly
+		//
+		sinsp_fdinfo_t* oldfdinfo = evt->m_tinfo->get_fd(retval);
+
+		if(oldfdinfo != NULL)
+		{
+			erase_fd_params eparams;
+
+			eparams.m_fd = retval;
+			eparams.m_fdinfo = oldfdinfo;
+			eparams.m_remove_from_table = false;
+			eparams.m_inspector = m_inspector;
+			eparams.m_tinfo = evt->m_tinfo;
+			eparams.m_ts = evt->get_ts();
+
+			erase_fd(&eparams);
+		}
+
 		//
 		// Add the new fd to the table.
-		// NOTE: dup2 and dup3 accept an existing FD and in that case they close it.
-		//       For us it's ok to just overwrite it.
 		//
 		evt->m_fdinfo = evt->m_tinfo->add_fd(retval, evt->m_fdinfo);
 	}
@@ -2882,3 +3233,120 @@ void sinsp_parser::parse_brk_munmap_mmap_exit(sinsp_evt* evt)
 		ASSERT(parinfo->m_len == sizeof(uint32_t));
 	}
 }
+
+void sinsp_parser::parse_setresuid_exit(sinsp_evt *evt)
+{
+	sinsp_evt_param *parinfo;
+	int64_t retval;
+	sinsp_evt *enter_evt = &m_tmp_evt;
+
+	//
+	// Extract the return value
+	//
+	parinfo = evt->get_param(0);
+	retval = *(int64_t *)parinfo->m_val;
+	ASSERT(parinfo->m_len == sizeof(int64_t));
+
+	if(retval >= 0 && retrieve_enter_event(enter_evt, evt))
+	{
+		parinfo = enter_evt->get_param(1);
+		ASSERT(parinfo->m_len == sizeof(uint32_t));
+		uint32_t new_euid = *(uint32_t *)parinfo->m_val;
+
+		if(new_euid < std::numeric_limits<uint32_t>::max())
+		{
+			evt->get_thread_info()->m_uid = new_euid;
+		}
+	}
+}
+
+void sinsp_parser::parse_setresgid_exit(sinsp_evt *evt)
+{
+	sinsp_evt_param *parinfo;
+	int64_t retval;
+	sinsp_evt *enter_evt = &m_tmp_evt;
+
+	//
+	// Extract the return value
+	//
+	parinfo = evt->get_param(0);
+	retval = *(int64_t *)parinfo->m_val;
+	ASSERT(parinfo->m_len == sizeof(int64_t));
+
+	if(retval >= 0 && retrieve_enter_event(enter_evt, evt))
+	{
+		parinfo = enter_evt->get_param(1);
+		ASSERT(parinfo->m_len == sizeof(uint32_t));
+		uint32_t new_egid = *(uint32_t *)parinfo->m_val;
+
+		if(new_egid < std::numeric_limits<uint32_t>::max())
+		{
+			evt->get_thread_info()->m_gid = new_egid;
+		}
+	}
+}
+
+void sinsp_parser::parse_setuid_exit(sinsp_evt *evt)
+{
+	sinsp_evt_param *parinfo;
+	int64_t retval;
+	sinsp_evt *enter_evt = &m_tmp_evt;
+
+	//
+	// Extract the return value
+	//
+	parinfo = evt->get_param(0);
+	retval = *(int64_t *)parinfo->m_val;
+	ASSERT(parinfo->m_len == sizeof(int64_t));
+
+	if(retval >= 0 && retrieve_enter_event(enter_evt, evt))
+	{
+		parinfo = enter_evt->get_param(0);
+		ASSERT(parinfo->m_len == sizeof(uint32_t));
+		uint32_t new_euid = *(uint32_t *)parinfo->m_val;
+		evt->get_thread_info()->m_uid = new_euid;
+	}
+}
+
+void sinsp_parser::parse_setgid_exit(sinsp_evt *evt)
+{
+	sinsp_evt_param *parinfo;
+	int64_t retval;
+	sinsp_evt *enter_evt = &m_tmp_evt;
+
+	//
+	// Extract the return value
+	//
+	parinfo = evt->get_param(0);
+	retval = *(int64_t *)parinfo->m_val;
+	ASSERT(parinfo->m_len == sizeof(int64_t));
+
+	if(retval >= 0 && retrieve_enter_event(enter_evt, evt))
+	{
+		parinfo = enter_evt->get_param(0);
+		ASSERT(parinfo->m_len == sizeof(uint32_t));
+		uint32_t new_egid = *(uint32_t *)parinfo->m_val;
+		evt->get_thread_info()->m_gid = new_egid;
+	}
+}
+
+void sinsp_parser::parse_container_evt(sinsp_evt *evt)
+{
+	sinsp_evt_param *parinfo;
+	sinsp_container_info container_info;
+
+	parinfo = evt->get_param(0);
+	container_info.m_id = parinfo->m_val;
+
+	parinfo = evt->get_param(1);
+	ASSERT(parinfo->m_len == sizeof(uint32_t));
+	container_info.m_type = (sinsp_container_type) *(uint32_t *)parinfo->m_val;
+
+	parinfo = evt->get_param(2);
+	container_info.m_name = parinfo->m_val;
+
+	parinfo = evt->get_param(3);
+	container_info.m_image = parinfo->m_val;
+
+	m_inspector->m_container_manager.add_container(container_info);
+}
