@@ -73,6 +73,7 @@ using namespace std;
 #include "stats.h"
 #include "ifinfo.h"
 #include "chisel.h"
+#include "container.h"
 
 #ifndef VISIBILITY_PRIVATE
 #define VISIBILITY_PRIVATE private:
@@ -125,8 +126,8 @@ public:
 	}
 
 	string m_name; ///< Field class name.
-	int32_t m_nfiedls; ///< Number of fields in this field group.
-	const filtercheck_field_info* m_fields; ///< Array containing m_nfiedls field descriptions.
+	int32_t m_nfields; ///< Number of fields in this field group.
+	const filtercheck_field_info* m_fields; ///< Array containing m_nfields field descriptions.
 	uint32_t m_flags;
 };
 
@@ -166,8 +167,7 @@ struct sinsp_capture_interrupt_exception : sinsp_exception
 /*!
   \brief The deafult way an event is converted to string by the library
 */
-//#define DEFAULT_OUTPUT_STR "*%evt.num %evt.time %evt.cpu %proc.name (%thread.tid) %evt.dir %evt.type %evt.args"
-#define DEFAULT_OUTPUT_STR "*%evt.time %evt.cpu %proc.name (%thread.tid) %evt.dir %evt.type %evt.args"
+#define DEFAULT_OUTPUT_STR "*%evt.num %evt.time %evt.cpu %proc.name (%thread.tid) %evt.dir %evt.type %evt.args"
 
 /** @defgroup inspector Main library
  @{
@@ -257,6 +257,24 @@ public:
 	*/
 	void set_snaplen(uint32_t snaplen);
 
+	/*!
+	  \brief Determine if this inspector is going to load user tables on
+	  startup.
+
+	  \param import_users if true, no user tables will be created for 
+	  this capture. This also means that no user or group info will be 
+	  written to the tracefile by the -w flag. The user/group tables are 
+	  necessary to use filter fields like user.name or group.name. However, 
+	  creating them can increase sysdig's startup time. Moreover, they contain
+	  information that could be privacy sensitive.
+
+	  \note default behavior is import_users=true.
+
+	  @throws a sinsp_exception containing the error string is thrown in case
+	   of failure.
+	*/
+	void set_import_users(bool import_users);
+
 	/*!
 	  \brief temporarily pauses event capture.
 
@@ -302,6 +320,12 @@ public:
 	*/
 	void set_log_callback(sinsp_logger_callback cb);
 
+	/*!
+	  \brief Specify the minimum severity of the messages that go into the logs
+	   emitted by the library.
+	*/
+	void set_min_log_severity(sinsp_logger::severity sev);
+
 	/*!
 	  \brief Start writing the captured events to file.
 
@@ -497,10 +521,38 @@ public:
 	*/
 	void set_fatfile_dump_mode(bool enable_fatfile);
 
+	/*!
+	  \brief Sets the max length of event argument strings. 
+
+	  \param len Max length after which an avent argument string is truncated.
+	   0 means no limit. Use this to reduce verbosity when printing event info
+	   on screen.
+	*/
+	void set_max_evt_output_len(uint32_t len);
+
 	/*!
 	  \brief Returns true if the debug mode is enabled.
 	*/
-	bool is_debug_enabled();
+	inline bool is_debug_enabled()
+	{
+		return m_isdebug_enabled;		
+	}
+
+        /*!
+          \brief Set a flag indicating if the command line requested to show container information.
+
+          \param set true if the command line arugment is set to show container information 
+        */
+        void set_print_container_data(bool print_container_data);
+
+
+	/*!
+	  \brief Returns true if the command line argument is set to show container information.
+	*/
+	inline bool is_print_container_data()
+	{
+		return m_print_container_data;
+	}
 
 	/*!
 	  \brief Lets a filter plugin request a protocol decoder.
@@ -535,7 +587,9 @@ public:
 	//
 	void stop_dropping_mode();
 	void start_dropping_mode(uint32_t sampling_ratio);
-	void import_ipv4_interface(const sinsp_ipv4_ifinfo& ifinfo);
+	void on_new_entry_from_proc(void* context, int64_t tid, scap_threadinfo* tinfo, 
+		scap_fdinfo* fdinfo, scap_t* newhandle);
+
 	//
 	// Allocates private state in the thread info class.
 	// Returns the ID to use when retrieving the memory area.
@@ -546,6 +600,7 @@ public:
 	sinsp_parser* get_parser();
 
 	bool setup_cycle_writer(string base_file_name, int rollover_mb, int duration_seconds, int file_limit, bool do_cycle, bool compress);
+	void import_ipv4_interface(const sinsp_ipv4_ifinfo& ifinfo);
 
 VISIBILITY_PRIVATE
 
@@ -562,6 +617,16 @@ private:
 
 	void add_thread(const sinsp_threadinfo& ptinfo);
 	void remove_thread(int64_t tid, bool force);
+	//
+	// Note: lookup_only should be used when the query for the thread is made
+	//       not as a consequence of an event for that thread arriving, but for
+	//       just for lookup reason. In that case, m_lastaccess_ts is not updated
+	//       and m_last_tinfo is not set.
+	//
+	inline sinsp_threadinfo* find_thread(int64_t tid, bool lookup_only);
+	// this is here for testing purposes only
+	sinsp_threadinfo* find_thread_test(int64_t tid, bool lookup_only);
+	bool remove_inactive_threads();
 
 	scap_t* m_h;
 	int64_t m_filesize;
@@ -569,6 +634,7 @@ private:
 	string m_input_filename;
 	bool m_isdebug_enabled;
 	bool m_isfatfile_enabled;
+	uint32_t m_max_evt_output_len;
 	bool m_compress;
 	sinsp_evt m_evt;
 	string m_lasterr;
@@ -588,6 +654,14 @@ private:
 
 	sinsp_thread_manager* m_thread_manager;
 
+	sinsp_container_manager m_container_manager;
+
+        //
+        // True if the command line argument is set to show container information
+	// The deafult is false set within the constructor
+        //
+	bool m_print_container_data;
+
 #ifdef HAS_FILTERING
 	uint64_t m_firstevent_ts;
 	sinsp_filter* m_filter;
@@ -603,6 +677,9 @@ private:
 	uint32_t m_n_proc_lookups;
 	uint32_t m_max_n_proc_lookups;
 	uint32_t m_max_n_proc_socket_lookups;
+#ifdef HAS_ANALYZER
+	vector<uint64_t> m_tid_collisions;
+#endif
 
 	//
 	// Saved snaplen
@@ -616,6 +693,11 @@ private:
 	uint64_t m_thread_timeout_ns;
 	uint64_t m_inactive_thread_scan_time_ns;
 
+	//
+	// Container limits
+	//
+	uint64_t m_inactive_container_scan_time_ns;
+
 	//
 	// How to render the data buffers
 	//
@@ -624,6 +706,7 @@ private:
 	//
 	// User and group tables
 	//
+	bool m_import_users;
 	unordered_map<uint32_t, scap_userinfo*> m_userlist;
 	unordered_map<uint32_t, scap_groupinfo*> m_grouplist;
 
@@ -633,16 +716,29 @@ private:
 	cycle_writer* m_cycle_writer;
 	bool m_write_cycling;
 
+#ifdef SIMULATE_DROP_MODE
 	//
 	// Some dropping infrastructure
 	//
 	bool m_isdropping;
+#endif
 
 	//
 	// Protocol decoding state
 	//
 	vector<sinsp_protodecoder*> m_decoders_reset_list;
 
+	//
+	// Meta event management
+	//
+	sinsp_evt m_meta_evt;
+	char* m_meta_evt_buf;
+	bool m_meta_evt_pending;
+
+#if defined(HAS_CAPTURE)
+	int64_t m_sysdig_pid;
+#endif
+
 	friend class sinsp_parser;
 	friend class sinsp_analyzer;
 	friend class sinsp_analyzer_parsers;
@@ -650,11 +746,14 @@ private:
 	friend class sinsp_threadinfo;
 	friend class sinsp_fdtable;
 	friend class sinsp_thread_manager;
+	friend class sinsp_container_manager;
 	friend class sinsp_dumper;
 	friend class sinsp_analyzer_fd_listener;
 	friend class sinsp_chisel;
 	friend class sinsp_protodecoder;
 	friend class lua_cbacks;
+	friend class sinsp_filter_check_container;
+	friend class sinsp_worker;
 
 	template<class TKey,class THash,class TCompare> friend class sinsp_connection_manager;
 };
