@@ -131,11 +131,12 @@ public:
 	int64_t m_tid;  ///< The id of this thread
 	int64_t m_pid; ///< The id of the process containing this thread. In single thread threads, this is equal to tid.
 	int64_t m_ptid; ///< The id of the process that started this thread.
-	int64_t m_progid; ///< Main program id. If this process is part of a logical group of processes (e.g. it's one of the apache processes), the tid of the process that is the head of this group.
 	string m_comm; ///< Command name (e.g. "top")
-	string m_exe; ///< Full command name (e.g. "/bin/top")
+	string m_exe; ///< argv[0] (e.g. "sshd: user@pts/4")
 	vector<string> m_args; ///< Command line arguments (e.g. "-d1")
 	vector<string> m_env; ///< Environment variables
+	vector<pair<string, string>> m_cgroups; ///< subsystem-cgroup pairs
+	string m_container_id; ///< heuristic-based container id
 	uint32_t m_flags; ///< The thread flags. See the PPM_CL_* declarations in ppm_events_public.h.
 	int64_t m_fdlimit;  ///< The maximum number of FDs this thread can open
 	uint32_t m_uid; ///< user id
@@ -146,6 +147,8 @@ public:
 	uint32_t m_vmswap_kb; ///< swapped memory (as kb).
 	uint64_t m_pfmajor; ///< number of major page faults since start.
 	uint64_t m_pfminor; ///< number of minor page faults since start.
+	int64_t m_vtid;  ///< The virtual id of this thread.
+	int64_t m_vpid; ///< The virtual id of the process containing this thread. In single thread threads, this is equal to vtid.
 
 	//
 	// State for multi-event processing
@@ -176,16 +179,29 @@ VISIBILITY_PRIVATE
 	void init(const scap_threadinfo* pi);
 	void fix_sockets_coming_from_proc();
 	sinsp_fdinfo_t* add_fd(int64_t fd, sinsp_fdinfo_t *fdinfo);
+	void add_fd(scap_fdinfo *fdinfo);
 	void remove_fd(int64_t fd);
 	sinsp_fdtable* get_fd_table();
 	void set_cwd(const char *cwd, uint32_t cwdlen);
 	sinsp_threadinfo* get_cwd_root();
 	void set_args(const char* args, size_t len);
 	void set_env(const char* env, size_t len);
+	void set_cgroups(const char* cgroups, size_t len);
 	void store_event(sinsp_evt *evt);
 	bool is_lastevent_data_valid();
-	void set_lastevent_data_validity(bool isvalid);
+	inline void set_lastevent_data_validity(bool isvalid)
+	{
+		if(isvalid)
+		{
+			m_lastevent_cpuid = (uint16_t)1;
+		}
+		else
+		{
+			m_lastevent_cpuid = (uint16_t) - 1;
+		}		
+	}
 	void allocate_private_state();
+	void compute_program_hash();
 
 	//  void push_fdop(sinsp_fdop* op);
 	// the queue of recent fd operations
@@ -198,13 +214,13 @@ VISIBILITY_PRIVATE
 	sinsp_fdtable m_fdtable; // The fd table of this thread
 	string m_cwd; // current working directory
 	sinsp_threadinfo* m_main_thread;
-	sinsp_threadinfo* m_main_program_thread;
 	uint8_t m_lastevent_data[SP_EVT_BUF_SIZE]; // Used by some event parsers to store the last enter event
 	vector<void*> m_private_state;
 
 	uint16_t m_lastevent_type;
 	uint16_t m_lastevent_cpuid;
 	sinsp_evt::category m_lastevent_category;
+	size_t m_program_hash;
 
 	friend class sinsp;
 	friend class sinsp_parser;
@@ -259,18 +275,11 @@ public:
 	void clear();
 
 	void set_listener(sinsp_threadtable_listener* listener);
-	//
-	// Note: lookup_only should be used when the query for the thread is made
-	//       not as a consequence of an event for that thread arriving, but for
-	//       just for lookup reason. In that case, m_lastaccess_ts is not updated
-	//       and m_last_tinfo is not set.
-	//
-	sinsp_threadinfo* get_thread(int64_t tid, bool lookup_only);
 	void add_thread(sinsp_threadinfo& threadinfo, bool from_scap_proctable);
 	void remove_thread(int64_t tid, bool force);
-	void remove_thread(threadinfo_map_iterator_t it, bool force);
 	// Returns true if the table is actually scanned
-	bool remove_inactive_threads();
+	// NOTE: this is implemented in sinsp.cpp so we can inline it from there
+	inline bool remove_inactive_threads();
 	void fix_sockets_coming_from_proc();
 	void reset_child_dependencies();
 	void create_child_dependencies();
@@ -291,10 +300,9 @@ public:
 	set<uint16_t> m_server_ports;
 
 private:
+	void remove_thread(threadinfo_map_iterator_t it, bool force);
 	void increment_mainthread_childcount(sinsp_threadinfo* threadinfo);
-	void increment_program_childcount(sinsp_threadinfo* threadinfo, uint32_t level, uint32_t notclosed_level);
-	// Don't set level, it's for internal use
-	void decrement_program_childcount(sinsp_threadinfo* threadinfo, uint32_t level = 0);
+	inline void clear_thread_pointers(threadinfo_map_iterator_t it);
 
 	sinsp* m_inspector;
 	threadinfo_map_t m_threadtable;
