@@ -96,7 +96,7 @@ static void usage()
 "                    run the specified chisel. If the chisel require arguments,\n"
 "                    they must be specified in the command line after the name.\n"
 " -cl, --list-chisels\n"
-"                    lists the available chisels. Looks for chisels in .,\n"
+"                    lists the available chisels. Looks for chisels in\n"
 "                    ./chisels, ~/.chisels and /usr/share/sysdig/chisels.\n"
 #endif
 #ifndef DISABLE_CGW
@@ -115,6 +115,14 @@ static void usage()
 "                    normally filtered before being analyzed, which is more\n"
 "                    efficient, but can cause state (e.g. FD names) to be lost.\n"
 " -D, --debug        Capture events about sysdig itself\n"
+" -E, --exclude-users\n"
+"                    Don't create the user/group tables by querying the OS when\n"
+"                    sysdig starts. This also means that no user or group info\n"
+"                    will be written to the tracefile by the -w flag.\n"
+"                    The user/group tables are necessary to use filter fields\n"
+"                    like user.name or group.name. However, creating them can\n"
+"                    increase sysdig's startup time. Moreover, they contain\n"
+"                    information that could be privacy sensitive.\n"
 " -F, --fatfile	     Enable fatfile mode\n"
 "                    when writing in fatfile mode, the output file will contain\n"
 "                    events that will be invisible when reading the file, but\n"
@@ -154,6 +162,7 @@ static void usage()
 " -P, --progress     Print progress on stderr while processing trace files\n"
 " -p <output_format>, --print=<output_format>\n"
 "                    Specify the format to be used when printing the events.\n"
+"                    With -pc or -pcontainer will use a container-friendly format.\n"
 "                    See the examples section below for more info.\n"
 " -q, --quiet        Don't print events on the screen\n"
 "                    Useful when dumping to disk.\n"
@@ -172,6 +181,12 @@ static void usage()
 "                    capture, d for delta between event enter and exit, and\n"
 "                    D for delta from the previous event.\n"
 " -v, --verbose      Verbose output.\n"
+"                    This flag will cause the full content of text and binary\n"
+"                    buffers to be printed on screen, instead of being truncated\n"
+"                    to 40 characters. Note that data buffers length is still\n"
+"                    limited by the snaplen (refer to the -s flag documentation)\n"
+"                    -v will also make sysdig print some summary information at\n"
+"                    the end of the capture.\n"
 " --version          Print version number.\n"
 " -w <writefile>, --write=<writefile>\n"
 "                    Write the captured events to <writefile>.\n"
@@ -196,7 +211,7 @@ static void usage()
 "Output format:\n\n"
 "By default, sysdig prints the information for each captured event on a single\n"
 " line with the following format:\n\n"
-" %%evt.num %%evt.time %%evt.cpu %%proc.name (%%thread.tid) %%evt.dir %%evt.type %%evt.args\n\n"
+" %%evt.num %%evt.time %%evt.cpu %%proc.name (%%thread.tid) %%evt.dir %%evt.type %%evt.info\n\n"
 "where:\n"
 " evt.num is the incremental event number\n"
 " evt.time is the event timestamp\n"
@@ -206,9 +221,11 @@ static void usage()
 "   PID for single thread processes\n"
 " evt.dir is the event direction, > for enter events and < for exit events\n"
 " evt.type is the name of the event, e.g. 'open' or 'read'\n"
-" evt.args is the list of event arguments.\n\n"
+" evt.info is the list of event arguments.\n\n"
 "The output format can be customized with the -p switch, using any of the\n"
 "fields listed by 'sysdig -l'.\n\n"
+"Using -pc or -pcontainer, the default format will be changed to a container-friendly one:\n\n"
+"%%evt.num %%evt.time %%evt.cpu %%container.name (%%container.id) %%proc.name (%%thread.tid:%%thread.vtid) %%evt.dir %%evt.type %%evt.info\n\n"
 "Examples:\n\n"
 " Capture all the events from the live system and print them to screen\n"
 "   $ sysdig\n\n"
@@ -603,6 +620,12 @@ captureinfo do_inspect(sinsp* inspector,
 				continue;
 			}
 
+			if(!inspector->is_debug_enabled() &&
+				ev->get_category() & EC_INTERNAL)
+			{
+				continue;
+			}
+
 			if(formatter->tostring(ev, &line))
 			{
 				//
@@ -678,9 +701,9 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 		{"chisel", required_argument, 0, 'c' },
 		{"list-chisels", no_argument, &cflag, 1 },
 #endif
-		{"compress", no_argument, 0, 'z' },
 		{"displayflt", no_argument, 0, 'd' },
 		{"debug", no_argument, 0, 'D'},
+		{"exclude-users", no_argument, 0, 'E' },
 		{"fatfile", no_argument, 0, 'F'},
 #ifndef DISABLE_CGW
 		{"seconds", required_argument, 0, 'G' },
@@ -711,16 +734,17 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 #endif
 		{"print-hex", no_argument, 0, 'x'},
 		{"print-hex-ascii", no_argument, 0, 'X'},
+		{"compress", no_argument, 0, 'z' },
 		{0, 0, 0, 0}
 	};
 
 	output_format = "*%evt.num <TIME> %evt.cpu %proc.name (%thread.tid) %evt.dir %evt.type %evt.info";
-//	output_format = DEFAULT_OUTPUT_STR;
 
 	try
 	{
 		inspector = new sinsp();
 
+
 #ifdef HAS_CHISELS
 		add_chisel_dirs(inspector);
 #endif
@@ -733,7 +757,7 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 #ifndef DISABLE_CGW
                                         "C:"
 #endif
-                                        "dDF"
+                                        "dDEF"
 #ifndef DISABLE_CGW
                                         "G:"
 #endif
@@ -826,6 +850,9 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 			case 'D':
 				inspector->set_debug_mode(true);
 				break;
+			case 'E':
+				inspector->set_import_users(false);
+				break;
 			case 'F':
 				inspector->set_fatfile_dump_mode(true);
 				break;
@@ -847,7 +874,6 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 			case 'i':
 				{
 					cname = optarg;
-
 					vector<chisel_desc> chlist;
 
 					sinsp_chisel::get_chisel_list(&chlist);
@@ -910,6 +936,18 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 					delete inspector;
 					return sysdig_init_res(EXIT_SUCCESS);
 				}
+				else if(string(optarg) == "c" || string(optarg) == "container")
+				{
+					output_format = "*%evt.num <TIME> %evt.cpu %container.name (%container.id) %proc.name (%thread.tid:%thread.vtid) %evt.dir %evt.type %evt.info";
+
+					//
+					// This enables chisels to determine if they should print container information
+					//
+					if ( inspector != NULL )
+                                        {
+                                           inspector->set_print_container_data( true );
+                                        }
+				}
 				else
 				{
 					output_format = optarg;
@@ -1015,6 +1053,11 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 			case 'z':
 				compress = true;
 				break;
+            // getopt_long : '?' for an ambiguous match or an extraneous parameter 
+			case '?':
+				delete inspector;
+				return sysdig_init_res(EXIT_FAILURE);
+				break;
 			default:
 				break;
 			}
@@ -1090,7 +1133,7 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 			for(int32_t j = optind + n_filterargs; j < argc; j++)
 			{
 				filter += argv[j];
-				if(j < argc)
+				if(j < argc - 1)
 				{
 					filter += " ";
 				}
@@ -1131,6 +1174,14 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 		//
 		sinsp_evt_formatter formatter(inspector, output_format);
 
+		//
+		// Set output buffers len
+		//
+		if(!verbose && g_chisels.size() == 0)
+		{
+			inspector->set_max_evt_output_len(80);
+		}
+
 		for(uint32_t j = 0; j < infiles.size() || infiles.size() == 0; j++)
 		{
 #ifdef HAS_FILTERING
@@ -1198,7 +1249,10 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 				{
 					open_success = true;
 
-					system("modprobe sysdig-probe > /dev/null 2> /dev/null");
+					if(system("modprobe sysdig-probe > /dev/null 2> /dev/null"))
+					{
+						fprintf(stderr, "Unable to load the driver\n");						
+					}
 
 					inspector->open("");
 				}
